@@ -122,11 +122,10 @@ class AccountAuditor(Daemon):
                 continue
             raise InvalidAccountInfo(_(
                 'The total %(key)s for the container (%(total)s) does not '
-                'match the sum of %(key)s across policies (%(sum)s)') % {
-                    'key': key,
-                    'total': info[key],
-                    'sum': policy_totals[key],
-                })
+                'match the sum of %(key)s across policies (%(sum)s)')
+                % {'key': key,
+                   'total': info[key],
+                   'sum': policy_totals[key]})
 
     def account_audit(self, path):
         """
@@ -141,12 +140,13 @@ class AccountAuditor(Daemon):
                 self.validate_per_policy_counts(broker)
                 self.logger.increment('passes')
                 self.account_passes += 1
-                self.logger.debug('Audit passed for %s' % broker)
+                self.logger.debug(_('Audit passed for %s'), broker)
         except InvalidAccountInfo as e:
             self.logger.increment('failures')
             self.account_failures += 1
             self.logger.error(
-                _('Audit Failed for %s: %s'), path, str(e))
+                _('Audit Failed for %(path)s: %(err)s'),
+                {'path': path, 'err': str(e)})
         except (Exception, Timeout):
             self.logger.increment('failures')
             self.account_failures += 1
