@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2016 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,11 @@
 """
 This StaticWeb WSGI middleware will serve container data as a static web site
 with index file and error file resolution and optional file listings. This mode
-is normally only active for anonymous requests. If you want to use it with
-authenticated requests, set the ``X-Web-Mode: true`` header on the request.
+is normally only active for anonymous requests. When using keystone for
+authentication set ``delay_auth_decision = true`` in the authtoken middleware
+configuration in your ``/etc/swift/proxy-server.conf`` file.  If you want to
+use it with authenticated requests, set the ``X-Web-Mode: true`` header on the
+request.
 
 The ``staticweb`` filter should be added to the pipeline in your
 ``/etc/swift/proxy-server.conf`` file just after any auth middleware. Also, the
@@ -28,25 +31,16 @@ added. For example::
     ...
 
     [pipeline:main]
-    pipeline = healthcheck cache tempauth staticweb proxy-server
+    pipeline = catch_errors healthcheck proxy-logging cache ratelimit tempauth
+               staticweb proxy-logging proxy-server
 
     ...
 
     [filter:staticweb]
     use = egg:swift#staticweb
-    # Seconds to cache container x-container-meta-web-* header values.
-    # cache_timeout = 300
-    # You can override the default log routing for this filter here:
-    # set log_name = staticweb
-    # set log_facility = LOG_LOCAL0
-    # set log_level = INFO
-    # set access_log_name = staticweb
-    # set access_log_facility = LOG_LOCAL0
-    # set access_log_level = INFO
-    # set log_headers = False
 
 Any publicly readable containers (for example, ``X-Container-Read: .r:*``, see
-`acls`_ for more information on this) will be checked for
+:ref:`acls` for more information on this) will be checked for
 X-Container-Meta-Web-Index and X-Container-Meta-Web-Error header values::
 
     X-Container-Meta-Web-Index  <index.name>
@@ -63,7 +57,7 @@ Unauthorized and 404 Not Found) will instead serve the
 ``X-Container-Meta-Web-Error: error.html`` will serve .../404error.html for
 requests for paths not found.
 
-For psuedo paths that have no <index.name>, this middleware can serve HTML file
+For pseudo paths that have no <index.name>, this middleware can serve HTML file
 listings if you set the ``X-Container-Meta-Web-Listings: true`` metadata item
 on the container.
 
@@ -74,6 +68,18 @@ the .../listing.css style sheet. If you "view source" in your browser on a
 listing page, you will see the well defined document structure that can be
 styled.
 
+By default, the listings will be rendered with a label of
+"Listing of /v1/account/container/path".  This can be altered by
+setting a ``X-Container-Meta-Web-Listings-Label: <label>``.  For example,
+if the label is set to "example.com", a label of
+"Listing of example.com/path" will be used instead.
+
+The content-type of directory marker objects can be modified by setting
+the ``X-Container-Meta-Web-Directory-Type`` header.  If the header is not set,
+application/directory is used by default.  Directory marker objects are
+0-byte objects that represent directories to create a simulated hierarchical
+structure.
+
 Example usage of this middleware via ``swift``:
 
     Make the container publicly readable::
@@ -92,6 +98,7 @@ Example usage of this middleware via ``swift``:
 
     Turn on listings::
 
+        swift post -r '.r:*,.rlistings' container
         swift post -m 'web-listings: true' container
 
     Now you should see object listings for paths and pseudo paths that have no
@@ -106,77 +113,53 @@ Example usage of this middleware via ``swift``:
         swift post -m 'web-error:error.html' container
 
     Now 401's should load 401error.html, 404's should load 404error.html, etc.
+
+    Set Content-Type of directory marker object::
+
+        swift post -m 'web-directory-type:text/directory' container
+
+    Now 0-byte objects with a content-type of text/directory will be treated
+    as directories rather than objects.
 """
 
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
-
 import cgi
+import json
 import time
-from urllib import unquote, quote as urllib_quote
 
-from webob import Response, Request
-from webob.exc import HTTPMovedPermanently, HTTPNotFound
-
-from swift.common.utils import cache_from_env, get_logger, human_readable, \
-                               split_path, TRUE_VALUES
+from swift.common.utils import human_readable, split_path, config_true_value, \
+    quote, register_swift_info, get_logger, urlparse
+from swift.common.wsgi import make_env, WSGIContext
+from swift.common.http import is_success, is_redirection, HTTP_NOT_FOUND
+from swift.common.swob import Response, HTTPMovedPermanently, HTTPNotFound, \
+    Request
+from swift.proxy.controllers.base import get_container_info
 
 
-def quote(value, safe='/'):
+class _StaticWebContext(WSGIContext):
     """
-    Patched version of urllib.quote that encodes utf-8 strings before quoting
-    """
-    if isinstance(value, unicode):
-        value = value.encode('utf-8')
-    return urllib_quote(value, safe)
+    The Static Web WSGI middleware filter; serves container data as a
+    static web site. See `staticweb`_ for an overview.
 
-
-class StaticWeb(object):
-    """
-    The Static Web WSGI middleware filter; serves container data as a static
-    web site. See `staticweb`_ for an overview.
-
-    :param app: The next WSGI application/filter in the paste.deploy pipeline.
-    :param conf: The filter configuration dict.
+    This _StaticWebContext is used by StaticWeb with each request
+    that might need to be handled to make keeping contextual
+    information about the request a bit simpler than storing it in
+    the WSGI env.
     """
 
-    def __init__(self, app, conf):
-        #: The next WSGI application/filter in the paste.deploy pipeline.
-        self.app = app
-        #: The filter configuration dict.
-        self.conf = conf
-        #: The seconds to cache the x-container-meta-web-* headers.,
-        self.cache_timeout = int(conf.get('cache_timeout', 300))
-        #: Logger for this filter.
-        self.logger = get_logger(conf, log_route='staticweb')
-        access_log_conf = {}
-        for key in ('log_facility', 'log_name', 'log_level'):
-            value = conf.get('access_' + key, conf.get(key, None))
-            if value:
-                access_log_conf[key] = value
-        #: Web access logger for this filter.
-        self.access_logger = get_logger(access_log_conf,
-                                        log_route='staticweb-access')
-        #: Indicates whether full HTTP headers should be logged or not.
-        self.log_headers = conf.get('log_headers') == 'True'
-        # Results from the last call to self._start_response.
-        self._response_status = None
-        self._response_headers = None
-        self._response_exc_info = None
+    def __init__(self, staticweb, version, account, container, obj):
+        WSGIContext.__init__(self, staticweb.app)
+        self.version = version
+        self.account = account
+        self.container = container
+        self.obj = obj
+        self.app = staticweb.app
+        self.url_scheme = staticweb.url_scheme
+        self.url_host = staticweb.url_host
+        self.agent = '%(orig)s StaticWeb'
         # Results from the last call to self._get_container_info.
-        self._index = self._error = self._listings = self._listings_css = None
-
-    def _start_response(self, status, headers, exc_info=None):
-        """
-        Saves response info without sending it to the remote client.
-        Uses the same semantics as the usual WSGI start_response.
-        """
-        self._response_status = status
-        self._response_headers = headers
-        self._response_exc_info = exc_info
+        self._index = self._error = self._listings = self._listings_css = \
+            self._dir_type = self._listings_label = None
 
     def _error_response(self, response, env, start_response):
         """
@@ -187,7 +170,6 @@ class StaticWeb(object):
         :param env: The original request WSGI environment.
         :param start_response: The WSGI start_response hook.
         """
-        self._log_response(env, self._get_status_int())
         if not self._error:
             start_response(self._response_status, self._response_headers,
                            self._response_exc_info)
@@ -195,12 +177,12 @@ class StaticWeb(object):
         save_response_status = self._response_status
         save_response_headers = self._response_headers
         save_response_exc_info = self._response_exc_info
-        tmp_env = self._get_escalated_env(env)
-        tmp_env['REQUEST_METHOD'] = 'GET'
-        tmp_env['PATH_INFO'] = '/%s/%s/%s/%s%s' % (self.version, self.account,
-            self.container, self._get_status_int(), self._error)
-        resp = self.app(tmp_env, self._start_response)
-        if self._get_status_int() // 100 == 2:
+        resp = self._app_call(make_env(
+            env, 'GET', '/%s/%s/%s/%s%s' % (
+                self.version, self.account, self.container,
+                self._get_status_int(), self._error),
+            self.agent, swift_source='SW'))
+        if is_success(self._get_status_int()):
             start_response(save_response_status, self._response_headers,
                            self._response_exc_info)
             return resp
@@ -208,68 +190,30 @@ class StaticWeb(object):
                        save_response_exc_info)
         return response
 
-    def _get_status_int(self):
-        """
-        Returns the HTTP status int from the last called self._start_response
-        result.
-        """
-        return int(self._response_status.split(' ', 1)[0])
-
-    def _get_escalated_env(self, env):
-        """
-        Returns a new fresh WSGI environment with escalated privileges to do
-        backend checks, listings, etc. that the remote user wouldn't be able to
-        accomplish directly.
-        """
-        new_env = {'REQUEST_METHOD': 'GET',
-            'HTTP_USER_AGENT': '%s StaticWeb' % env.get('HTTP_USER_AGENT')}
-        for name in ('eventlet.posthooks', 'swift.trans_id', 'REMOTE_USER',
-                     'SCRIPT_NAME', 'SERVER_NAME', 'SERVER_PORT',
-                     'SERVER_PROTOCOL', 'swift.cache'):
-            if name in env:
-                new_env[name] = env[name]
-        return new_env
-
-    def _get_container_info(self, env, start_response):
+    def _get_container_info(self, env):
         """
         Retrieves x-container-meta-web-index, x-container-meta-web-error,
-        x-container-meta-web-listings, and x-container-meta-web-listings-css
-        from memcache or from the cluster and stores the result in memcache and
-        in self._index, self._error, self._listings, and self._listings_css.
+        x-container-meta-web-listings, x-container-meta-web-listings-css,
+        and x-container-meta-web-directory-type from memcache or from the
+        cluster and stores the result in memcache and in self._index,
+        self._error, self._listings, self._listings_css and self._dir_type.
 
         :param env: The WSGI environment dict.
-        :param start_response: The WSGI start_response hook.
+        :return container_info: The container_info dict.
         """
-        self._index = self._error = self._listings = self._listings_css = None
-        memcache_client = cache_from_env(env)
-        if memcache_client:
-            memcache_key = '/staticweb/%s/%s/%s' % (self.version, self.account,
-                                                    self.container)
-            cached_data = memcache_client.get(memcache_key)
-            if cached_data:
-                (self._index, self._error, self._listings,
-                 self._listings_css) = cached_data
-                return
-        tmp_env = self._get_escalated_env(env)
-        tmp_env['REQUEST_METHOD'] = 'HEAD'
-        req = Request.blank('/%s/%s/%s' % (self.version, self.account,
-            self.container), environ=tmp_env)
-        resp = req.get_response(self.app)
-        if resp.status_int // 100 == 2:
-            self._index = \
-                resp.headers.get('x-container-meta-web-index', '').strip()
-            self._error = \
-                resp.headers.get('x-container-meta-web-error', '').strip()
-            self._listings = \
-                resp.headers.get('x-container-meta-web-listings', '').strip()
-            self._listings_css = \
-                resp.headers.get('x-container-meta-web-listings-css',
-                                 '').strip()
-            if memcache_client:
-                memcache_client.set(memcache_key,
-                    (self._index, self._error, self._listings,
-                     self._listings_css),
-                    timeout=self.cache_timeout)
+        self._index = self._error = self._listings = self._listings_css = \
+            self._dir_type = None
+        container_info = get_container_info(
+            env, self.app, swift_source='SW')
+        if is_success(container_info['status']):
+            meta = container_info.get('meta', {})
+            self._index = meta.get('web-index', '').strip()
+            self._error = meta.get('web-error', '').strip()
+            self._listings = meta.get('web-listings', '').strip()
+            self._listings_label = meta.get('web-listings-label', '').strip()
+            self._listings_css = meta.get('web-listings-css', '').strip()
+            self._dir_type = meta.get('web-directory-type', '').strip()
+        return container_info
 
     def _listing(self, env, start_response, prefix=None):
         """
@@ -279,35 +223,67 @@ class StaticWeb(object):
         :param start_response: The original WSGI start_response hook.
         :param prefix: Any prefix desired for the container listing.
         """
-        if self._listings.lower() not in TRUE_VALUES:
-            resp = HTTPNotFound()(env, self._start_response)
+        label = env['PATH_INFO']
+        if self._listings_label:
+            groups = env['PATH_INFO'].split('/')
+            label = '{0}/{1}'.format(self._listings_label,
+                                     '/'.join(groups[4:]))
+
+        if not config_true_value(self._listings):
+            body = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 ' \
+                'Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' \
+                '<html>\n' \
+                '<head>\n' \
+                '<title>Listing of %s</title>\n' % cgi.escape(label)
+            if self._listings_css:
+                body += '  <link rel="stylesheet" type="text/css" ' \
+                    'href="%s" />\n' % self._build_css_path(prefix or '')
+            else:
+                body += '  <style type="text/css">\n' \
+                    '   h1 {font-size: 1em; font-weight: bold;}\n' \
+                    '   p {font-size: 2}\n' \
+                    '  </style>\n'
+            body += '</head>\n<body>' \
+                '  <h1>Web Listing Disabled</h1>' \
+                '   <p>The owner of this web site has disabled web listing.' \
+                '   <p>If you are the owner of this web site, you can enable' \
+                '   web listing by setting X-Container-Meta-Web-Listings.</p>'
+            if self._index:
+                body += '<h1>Index File Not Found</h1>' \
+                    ' <p>The owner of this web site has set ' \
+                    ' <b>X-Container-Meta-Web-Index: %s</b>. ' \
+                    ' However, this file is not found.</p>' % self._index
+            body += ' </body>\n</html>\n'
+            resp = HTTPNotFound(body=body)(env, self._start_response)
             return self._error_response(resp, env, start_response)
-        tmp_env = self._get_escalated_env(env)
-        tmp_env['REQUEST_METHOD'] = 'GET'
-        tmp_env['PATH_INFO'] = \
-            '/%s/%s/%s' % (self.version, self.account, self.container)
+        tmp_env = make_env(
+            env, 'GET', '/%s/%s/%s' % (
+                self.version, self.account, self.container),
+            self.agent, swift_source='SW')
         tmp_env['QUERY_STRING'] = 'delimiter=/&format=json'
         if prefix:
             tmp_env['QUERY_STRING'] += '&prefix=%s' % quote(prefix)
         else:
             prefix = ''
-        resp = self.app(tmp_env, self._start_response)
-        if self._get_status_int() // 100 != 2:
+        resp = self._app_call(tmp_env)
+        if not is_success(self._get_status_int()):
             return self._error_response(resp, env, start_response)
-        listing = json.loads(''.join(resp))
+        listing = None
+        body = ''.join(resp)
+        if body:
+            listing = json.loads(body)
         if not listing:
             resp = HTTPNotFound()(env, self._start_response)
             return self._error_response(resp, env, start_response)
         headers = {'Content-Type': 'text/html; charset=UTF-8'}
         body = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 ' \
-                'Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' \
+               'Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' \
                '<html>\n' \
                ' <head>\n' \
-               '  <title>Listing of %s</title>\n' % \
-               cgi.escape(env['PATH_INFO'])
+               '  <title>Listing of %s</title>\n' % cgi.escape(label)
         if self._listings_css:
             body += '  <link rel="stylesheet" type="text/css" ' \
-                        'href="%s" />\n' % (self._build_css_path(prefix))
+                    'href="%s" />\n' % (self._build_css_path(prefix))
         else:
             body += '  <style type="text/css">\n' \
                     '   h1 {font-size: 1em; font-weight: bold;}\n' \
@@ -323,8 +299,7 @@ class StaticWeb(object):
                 '    <th class="colname">Name</th>\n' \
                 '    <th class="colsize">Size</th>\n' \
                 '    <th class="coldate">Date</th>\n' \
-                '   </tr>\n' % \
-                cgi.escape(env['PATH_INFO'])
+                '   </tr>\n' % cgi.escape(label)
         if prefix:
             body += '   <tr id="parent" class="item">\n' \
                     '    <td class="colname"><a href="../">../</a></td>\n' \
@@ -333,7 +308,7 @@ class StaticWeb(object):
                     '   </tr>\n'
         for item in listing:
             if 'subdir' in item:
-                subdir = item['subdir']
+                subdir = item['subdir'].encode("utf-8")
                 if prefix:
                     subdir = subdir[len(prefix):]
                 body += '   <tr class="item subdir">\n' \
@@ -344,25 +319,27 @@ class StaticWeb(object):
                         (quote(subdir), cgi.escape(subdir))
         for item in listing:
             if 'name' in item:
-                name = item['name']
+                name = item['name'].encode("utf-8")
                 if prefix:
                     name = name[len(prefix):]
+                content_type = item['content_type'].encode("utf-8")
+                bytes = human_readable(item['bytes'])
+                last_modified = (
+                    cgi.escape(item['last_modified'].encode("utf-8")).
+                    split('.')[0].replace('T', ' '))
                 body += '   <tr class="item %s">\n' \
                         '    <td class="colname"><a href="%s">%s</a></td>\n' \
                         '    <td class="colsize">%s</td>\n' \
                         '    <td class="coldate">%s</td>\n' \
                         '   </tr>\n' % \
                         (' '.join('type-' + cgi.escape(t.lower(), quote=True)
-                                  for t in item['content_type'].split('/')),
+                                  for t in content_type.split('/')),
                          quote(name), cgi.escape(name),
-                         human_readable(item['bytes']),
-                         cgi.escape(item['last_modified']).split('.')[0].
-                            replace('T', ' '))
+                         bytes, last_modified)
         body += '  </table>\n' \
                 ' </body>\n' \
                 '</html>\n'
         resp = Response(headers=headers, body=body)
-        self._log_response(env, resp.status_int)
         return resp(env, start_response)
 
     def _build_css_path(self, prefix=''):
@@ -378,40 +355,57 @@ class StaticWeb(object):
             css_path = '../' * prefix.count('/') + quote(self._listings_css)
         return css_path
 
-    def _handle_container(self, env, start_response):
+    def _redirect_with_slash(self, env_, start_response):
+        env = {}
+        env.update(env_)
+        if self.url_scheme:
+            env['wsgi.url_scheme'] = self.url_scheme
+        if self.url_host:
+            env['HTTP_HOST'] = self.url_host
+        resp = HTTPMovedPermanently(location=(env['PATH_INFO'] + '/'))
+        return resp(env, start_response)
+
+    def handle_container(self, env, start_response):
         """
         Handles a possible static web request for a container.
 
         :param env: The original WSGI environment dict.
         :param start_response: The original WSGI start_response hook.
         """
-        self._get_container_info(env, start_response)
+        container_info = self._get_container_info(env)
+        req = Request(env)
+        req.acl = container_info['read_acl']
+        # we checked earlier that swift.authorize is set in env
+        aresp = env['swift.authorize'](req)
+        if aresp:
+            resp = aresp(env, self._start_response)
+            return self._error_response(resp, env, start_response)
+
         if not self._listings and not self._index:
-            if env.get('HTTP_X_WEB_MODE', 'f').lower() in TRUE_VALUES:
+            if config_true_value(env.get('HTTP_X_WEB_MODE', 'f')):
                 return HTTPNotFound()(env, start_response)
             return self.app(env, start_response)
-        if env['PATH_INFO'][-1] != '/':
-            resp = HTTPMovedPermanently(
-                location=(env['PATH_INFO'] + '/'))
-            self._log_response(env, resp.status_int)
-            return resp(env, start_response)
+        if not env['PATH_INFO'].endswith('/'):
+            return self._redirect_with_slash(env, start_response)
         if not self._index:
             return self._listing(env, start_response)
         tmp_env = dict(env)
         tmp_env['HTTP_USER_AGENT'] = \
             '%s StaticWeb' % env.get('HTTP_USER_AGENT')
+        tmp_env['swift.source'] = 'SW'
         tmp_env['PATH_INFO'] += self._index
-        resp = self.app(tmp_env, self._start_response)
+        resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
-        if status_int == 404:
+        if status_int == HTTP_NOT_FOUND:
             return self._listing(env, start_response)
-        elif self._get_status_int() // 100 not in (2, 3):
+        elif not is_success(self._get_status_int()) and \
+                not is_redirection(self._get_status_int()):
             return self._error_response(resp, env, start_response)
         start_response(self._response_status, self._response_headers,
                        self._response_exc_info)
         return resp
 
-    def _handle_object(self, env, start_response):
+    def handle_object(self, env, start_response):
         """
         Handles a possible static web request for an object. This object could
         resolve into an index or listing request.
@@ -422,55 +416,96 @@ class StaticWeb(object):
         tmp_env = dict(env)
         tmp_env['HTTP_USER_AGENT'] = \
             '%s StaticWeb' % env.get('HTTP_USER_AGENT')
-        resp = self.app(tmp_env, self._start_response)
+        tmp_env['swift.source'] = 'SW'
+        resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
-        if status_int // 100 in (2, 3):
+        self._get_container_info(env)
+        if is_success(status_int) or is_redirection(status_int):
+            # Treat directory marker objects as not found
+            if not self._dir_type:
+                self._dir_type = 'application/directory'
+            content_length = self._response_header_value('content-length')
+            content_length = int(content_length) if content_length else 0
+            if self._response_header_value('content-type') == self._dir_type \
+                    and content_length <= 1:
+                status_int = HTTP_NOT_FOUND
+            else:
+                start_response(self._response_status, self._response_headers,
+                               self._response_exc_info)
+                return resp
+        if status_int != HTTP_NOT_FOUND:
+            # Retaining the previous code's behavior of not using custom error
+            # pages for non-404 errors.
+            self._error = None
+            return self._error_response(resp, env, start_response)
+        if not self._listings and not self._index:
             start_response(self._response_status, self._response_headers,
                            self._response_exc_info)
             return resp
-        if status_int != 404:
-            return self._error_response(resp, env, start_response)
-        self._get_container_info(env, start_response)
-        if not self._listings and not self._index:
-            return self.app(env, start_response)
-        status_int = 404
+        status_int = HTTP_NOT_FOUND
         if self._index:
             tmp_env = dict(env)
             tmp_env['HTTP_USER_AGENT'] = \
                 '%s StaticWeb' % env.get('HTTP_USER_AGENT')
-            if tmp_env['PATH_INFO'][-1] != '/':
+            tmp_env['swift.source'] = 'SW'
+            if not tmp_env['PATH_INFO'].endswith('/'):
                 tmp_env['PATH_INFO'] += '/'
             tmp_env['PATH_INFO'] += self._index
-            resp = self.app(tmp_env, self._start_response)
+            resp = self._app_call(tmp_env)
             status_int = self._get_status_int()
-            if status_int // 100 in (2, 3):
-                if env['PATH_INFO'][-1] != '/':
-                    resp = HTTPMovedPermanently(
-                        location=env['PATH_INFO'] + '/')
-                    self._log_response(env, resp.status_int)
-                    return resp(env, start_response)
+            if is_success(status_int) or is_redirection(status_int):
+                if not env['PATH_INFO'].endswith('/'):
+                    return self._redirect_with_slash(env, start_response)
                 start_response(self._response_status, self._response_headers,
                                self._response_exc_info)
                 return resp
-        if status_int == 404:
-            if env['PATH_INFO'][-1] != '/':
-                tmp_env = self._get_escalated_env(env)
-                tmp_env['REQUEST_METHOD'] = 'GET'
-                tmp_env['PATH_INFO'] = '/%s/%s/%s' % (self.version,
-                    self.account, self.container)
+        if status_int == HTTP_NOT_FOUND:
+            if not env['PATH_INFO'].endswith('/'):
+                tmp_env = make_env(
+                    env, 'GET', '/%s/%s/%s' % (
+                        self.version, self.account, self.container),
+                    self.agent, swift_source='SW')
                 tmp_env['QUERY_STRING'] = 'limit=1&format=json&delimiter' \
                     '=/&limit=1&prefix=%s' % quote(self.obj + '/')
-                resp = self.app(tmp_env, self._start_response)
-                if self._get_status_int() // 100 != 2 or \
-                        not json.loads(''.join(resp)):
+                resp = self._app_call(tmp_env)
+                body = ''.join(resp)
+                if not is_success(self._get_status_int()) or not body or \
+                        not json.loads(body):
                     resp = HTTPNotFound()(env, self._start_response)
                     return self._error_response(resp, env, start_response)
-                resp = HTTPMovedPermanently(location=env['PATH_INFO'] +
-                    '/')
-                self._log_response(env, resp.status_int)
-                return resp(env, start_response)
+                return self._redirect_with_slash(env, start_response)
             return self._listing(env, start_response, self.obj)
 
+
+class StaticWeb(object):
+    """
+    The Static Web WSGI middleware filter; serves container data as a static
+    web site. See `staticweb`_ for an overview.
+
+    The proxy logs created for any subrequests made will have swift.source set
+    to "SW".
+
+    :param app: The next WSGI application/filter in the paste.deploy pipeline.
+    :param conf: The filter configuration dict.
+    """
+
+    def __init__(self, app, conf):
+        #: The next WSGI application/filter in the paste.deploy pipeline.
+        self.app = app
+        #: The filter configuration dict. Only used in tests.
+        self.conf = conf
+        self.logger = get_logger(conf, log_route='staticweb')
+
+        # We expose a more general "url_base" parameter in case we want
+        # to incorporate the path prefix later. Currently it is discarded.
+        url_base = conf.get('url_base', None)
+        self.url_scheme = None
+        self.url_host = None
+        if url_base:
+            parsed = urlparse(url_base)
+            self.url_scheme = parsed.scheme
+            self.url_host = parsed.netloc
+
     def __call__(self, env, start_response):
         """
         Main hook into the WSGI paste.deploy filter/app pipeline.
@@ -479,93 +514,34 @@ class StaticWeb(object):
         :param start_response: The WSGI start_response hook.
         """
         env['staticweb.start_time'] = time.time()
+        if 'swift.authorize' not in env:
+            self.logger.warning(
+                'No authentication middleware authorized request yet. '
+                'Skipping staticweb')
+            return self.app(env, start_response)
         try:
-            (self.version, self.account, self.container, self.obj) = \
+            (version, account, container, obj) = \
                 split_path(env['PATH_INFO'], 2, 4, True)
         except ValueError:
             return self.app(env, start_response)
-        memcache_client = cache_from_env(env)
-        if memcache_client:
-            if env['REQUEST_METHOD'] in ('PUT', 'POST'):
-                if not self.obj and self.container:
-                    memcache_key = '/staticweb/%s/%s/%s' % \
-                        (self.version, self.account, self.container)
-                    memcache_client.delete(memcache_key)
-                return self.app(env, start_response)
-        if (env['REQUEST_METHOD'] not in ('HEAD', 'GET') or
-            (env.get('REMOTE_USER') and
-             env.get('HTTP_X_WEB_MODE', 'f').lower() not in TRUE_VALUES) or
-            (not env.get('REMOTE_USER') and
-             env.get('HTTP_X_WEB_MODE', 't').lower() not in TRUE_VALUES)):
+        if env['REQUEST_METHOD'] not in ('HEAD', 'GET'):
             return self.app(env, start_response)
-        if self.obj:
-            return self._handle_object(env, start_response)
-        elif self.container:
-            return self._handle_container(env, start_response)
-        return self.app(env, start_response)
-
-    def _log_response(self, env, status_int):
-        """
-        Logs an access line for StaticWeb responses; use when the next app in
-        the pipeline will not be handling the final response to the remote
-        user.
-
-        Assumes that the request and response bodies are 0 bytes or very near 0
-        so no bytes transferred are tracked or logged.
-
-        This does mean that the listings responses that actually do transfer
-        content will not be logged with any bytes transferred, but in counter
-        to that the full bytes for the underlying listing will be logged by the
-        proxy even if the remote client disconnects early for the StaticWeb
-        listing.
-
-        I didn't think the extra complexity of getting the bytes transferred
-        exactly correct for these requests was worth it, but perhaps someone
-        else will think it is.
-
-        To get things exact, this filter would need to use an
-        eventlet.posthooks logger like the proxy does and any log processing
-        systems would need to ignore some (but not all) proxy requests made by
-        StaticWeb if they were just interested in the bytes transferred to the
-        remote client.
-        """
-        trans_time = '%.4f' % (time.time() -
-                               env.get('staticweb.start_time', time.time()))
-        the_request = quote(unquote(env['PATH_INFO']))
-        if env.get('QUERY_STRING'):
-            the_request = the_request + '?' + env['QUERY_STRING']
-        # remote user for zeus
-        client = env.get('HTTP_X_CLUSTER_CLIENT_IP')
-        if not client and 'HTTP_X_FORWARDED_FOR' in env:
-            # remote user for other lbs
-            client = env['HTTP_X_FORWARDED_FOR'].split(',')[0].strip()
-        logged_headers = None
-        if self.log_headers:
-            logged_headers = '\n'.join('%s: %s' % (k, v)
-                for k, v in req.headers.items())
-        self.access_logger.info(' '.join(quote(str(x)) for x in (
-            client or '-',
-            env.get('REMOTE_ADDR', '-'),
-            time.strftime('%d/%b/%Y/%H/%M/%S', time.gmtime()),
-            env['REQUEST_METHOD'],
-            the_request,
-            env['SERVER_PROTOCOL'],
-            status_int,
-            env.get('HTTP_REFERER', '-'),
-            env.get('HTTP_USER_AGENT', '-'),
-            env.get('HTTP_X_AUTH_TOKEN', '-'),
-            '-',
-            '-',
-            env.get('HTTP_ETAG', '-'),
-            env.get('swift.trans_id', '-'),
-            logged_headers or '-',
-            trans_time)))
+        if env.get('REMOTE_USER') and \
+                not config_true_value(env.get('HTTP_X_WEB_MODE', 'f')):
+            return self.app(env, start_response)
+        if not container:
+            return self.app(env, start_response)
+        context = _StaticWebContext(self, version, account, container, obj)
+        if obj:
+            return context.handle_object(env, start_response)
+        return context.handle_container(env, start_response)
 
 
 def filter_factory(global_conf, **local_conf):
-    """ Returns a Static Web WSGI filter for use with paste.deploy. """
+    """Returns a Static Web WSGI filter for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
+    register_swift_info('staticweb')
 
     def staticweb_filter(app):
         return StaticWeb(app, conf)
