@@ -25,10 +25,9 @@ from swift.common.direct_client import (
     direct_head_container, direct_delete_container_object,
     direct_put_container_object, ClientException)
 from swift.common.internal_client import InternalClient, UnexpectedResponse
-from swift.common.utils import get_logger, split_path, quorum_size, \
+from swift.common.utils import get_logger, split_path, majority_size, \
     FileLikeIter, Timestamp, last_modified_date_to_timestamp, \
-    LRUCache
-
+    LRUCache, decode_timestamps
 
 MISPLACED_OBJECTS_ACCOUNT = '.misplaced_objects'
 MISPLACED_OBJECTS_CONTAINER_DIVISOR = 3600  # 1 hour
@@ -116,7 +115,18 @@ def best_policy_index(headers):
 
 
 def get_reconciler_container_name(obj_timestamp):
-    return str(int(Timestamp(obj_timestamp)) //
+    """
+    Get the name of a container into which a misplaced object should be
+    enqueued. The name is the object's last modified time rounded down to the
+    nearest hour.
+
+    :param obj_timestamp: a string representation of the object's 'created_at'
+                          time from it's container db row.
+    :return: a container name
+    """
+    # Use last modified time of object to determine reconciler container name
+    _junk, _junk, ts_meta = decode_timestamps(obj_timestamp)
+    return str(int(ts_meta) //
                MISPLACED_OBJECTS_CONTAINER_DIVISOR *
                MISPLACED_OBJECTS_CONTAINER_DIVISOR)
 
@@ -184,7 +194,7 @@ def add_to_reconciler_queue(container_ring, account, container, obj,
                              server
 
     :returns: .misplaced_object container name, False on failure. "Success"
-              means a quorum of containers got the update.
+              means a majority of containers got the update.
     """
     container_name = get_reconciler_container_name(obj_timestamp)
     object_name = get_reconciler_obj_name(obj_policy_index, account,
@@ -222,7 +232,7 @@ def add_to_reconciler_queue(container_ring, account, container, obj,
                    response_timeout=response_timeout)
 
     successes = sum(pile)
-    if successes >= quorum_size(len(nodes)):
+    if successes >= majority_size(len(nodes)):
         return container_name
     else:
         return False
@@ -262,7 +272,7 @@ def parse_raw_obj(obj_info):
         'container': container,
         'obj': obj,
         'q_op': q_op,
-        'q_ts': Timestamp(obj_info['hash']),
+        'q_ts': decode_timestamps((obj_info['hash']))[0],
         'q_record': last_modified_date_to_timestamp(
             obj_info['last_modified']),
         'path': '/%s/%s/%s' % (account, container, obj)
@@ -279,7 +289,7 @@ def direct_get_container_policy_index(container_ring, account_name,
     :param container_ring: ring in which to look up the container locations
     :param account_name: name of the container's account
     :param container_name: name of the container
-    :returns: storage policy index, or None if it couldn't get a quorum
+    :returns: storage policy index, or None if it couldn't get a majority
     """
     def _eat_client_exception(*args):
         try:
@@ -297,7 +307,7 @@ def direct_get_container_policy_index(container_ring, account_name,
                    container_name)
 
     headers = [x for x in pile if x is not None]
-    if len(headers) < quorum_size(len(nodes)):
+    if len(headers) < majority_size(len(nodes)):
         return
     return best_policy_index(headers)
 
