@@ -13,15 +13,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 import os
 import time
 import traceback
+import math
 from swift import gettext_ as _
 from xml.etree.cElementTree import Element, SubElement, tostring
 
 from eventlet import Timeout
 
 import swift.common.db
+from swift.container.sync_store import ContainerSyncStore
 from swift.container.backend import ContainerBroker, DATADIR
 from swift.container.replicator import ContainerReplicatorRpc
 from swift.common.db import DatabaseAlreadyExists
@@ -30,7 +33,7 @@ from swift.common.request_helpers import get_param, get_listing_content_type, \
     split_and_validate_path, is_sys_or_user_meta
 from swift.common.utils import get_logger, hash_path, public, \
     Timestamp, storage_directory, validate_sync_to, \
-    config_true_value, json, timing_stats, replication, \
+    config_true_value, timing_stats, replication, \
     override_bytes_from_content_type, get_log_line
 from swift.common.constraints import check_mount, valid_timestamp, check_utf8
 from swift.common import constraints
@@ -38,10 +41,12 @@ from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout
 from swift.common.http import HTTP_NOT_FOUND, is_success
 from swift.common.storage_policy import POLICIES
+from swift.common.base_storage_server import BaseStorageServer
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPConflict, \
     HTTPCreated, HTTPInternalServerError, HTTPNoContent, HTTPNotFound, \
     HTTPPreconditionFailed, HTTPMethodNotAllowed, Request, Response, \
-    HTTPInsufficientStorage, HTTPException, HeaderKeyDict
+    HTTPInsufficientStorage, HTTPException
 
 
 def gen_resp_headers(info, is_deleted=False):
@@ -71,24 +76,22 @@ def gen_resp_headers(info, is_deleted=False):
     return headers
 
 
-class ContainerController(object):
+class ContainerController(BaseStorageServer):
     """WSGI Controller for the container server."""
 
     # Ensure these are all lowercase
     save_headers = ['x-container-read', 'x-container-write',
                     'x-container-sync-key', 'x-container-sync-to']
+    server_type = 'container-server'
 
     def __init__(self, conf, logger=None):
+        super(ContainerController, self).__init__(conf)
         self.logger = logger or get_logger(conf, log_route='container-server')
         self.log_requests = config_true_value(conf.get('log_requests', 'true'))
         self.root = conf.get('devices', '/srv/node')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
-        self.node_timeout = int(conf.get('node_timeout', 3))
+        self.node_timeout = float(conf.get('node_timeout', 3))
         self.conn_timeout = float(conf.get('conn_timeout', 0.5))
-        replication_server = conf.get('replication_server', None)
-        if replication_server is not None:
-            replication_server = config_true_value(replication_server)
-        self.replication_server = replication_server
         #: ContainerSyncCluster instance for validating sync-to values.
         self.realms_conf = ContainerSyncRealms(
             os.path.join(
@@ -110,6 +113,9 @@ class ContainerController(object):
             self.save_headers.append('x-versions-location')
         swift.common.db.DB_PREALLOCATION = \
             config_true_value(conf.get('db_preallocation', 'f'))
+        self.sync_store = ContainerSyncStore(self.root,
+                                             self.logger,
+                                             self.mount_check)
 
     def _get_container_broker(self, drive, part, account, container, **kwargs):
         """
@@ -178,15 +184,20 @@ class ContainerController(object):
         if len(account_hosts) != len(account_devices):
             # This shouldn't happen unless there's a bug in the proxy,
             # but if there is, we want to know about it.
-            self.logger.error(_('ERROR Account update failed: different  '
-                                'numbers of hosts and devices in request: '
-                                '"%s" vs "%s"') %
-                               (req.headers.get('X-Account-Host', ''),
-                                req.headers.get('X-Account-Device', '')))
+            self.logger.error(_(
+                'ERROR Account update failed: different  '
+                'numbers of hosts and devices in request: '
+                '"%(hosts)s" vs "%(devices)s"') % {
+                    'hosts': req.headers.get('X-Account-Host', ''),
+                    'devices': req.headers.get('X-Account-Device', '')})
             return HTTPBadRequest(req=req)
 
         if account_partition:
-            updates = zip(account_hosts, account_devices)
+            # zip is lazy on py3, but we need a list, so force evaluation.
+            # On py2 it's an extra list copy, but the list is so small
+            # (one element per replica in account ring, usually 3) that it
+            # doesn't matter.
+            updates = list(zip(account_hosts, account_devices))
         else:
             updates = []
 
@@ -238,6 +249,13 @@ class ContainerController(object):
         else:
             return None
 
+    def _update_sync_store(self, broker, method):
+        try:
+            self.sync_store.update_sync_store(broker)
+        except Exception:
+            self.logger.exception('Failed to update sync_store %s during %s' %
+                                  (broker.db_file, method))
+
     @public
     @timing_stats()
     def DELETE(self, req):
@@ -272,6 +290,7 @@ class ContainerController(object):
             broker.delete_db(req_timestamp.internal)
             if not broker.is_deleted():
                 return HTTPConflict(request=req)
+            self._update_sync_store(broker, 'DELETE')
             resp = self.account_update(req, account, container, broker)
             if resp:
                 return resp
@@ -309,7 +328,9 @@ class ContainerController(object):
         elif requested_policy_index is not None:
             # validate requested policy with existing container
             if requested_policy_index != broker.storage_policy_index:
-                raise HTTPConflict(request=req)
+                raise HTTPConflict(request=req,
+                                   headers={'x-backend-storage-policy-index':
+                                            broker.storage_policy_index})
         broker.update_put_timestamp(timestamp)
         if broker.is_deleted():
             raise HTTPConflict(request=req)
@@ -350,7 +371,9 @@ class ContainerController(object):
                               int(req.headers['x-size']),
                               req.headers['x-content-type'],
                               req.headers['x-etag'], 0,
-                              obj_policy_index)
+                              obj_policy_index,
+                              req.headers.get('x-content-type-timestamp'),
+                              req.headers.get('x-meta-timestamp'))
             return HTTPCreated(request=req)
         else:   # put container
             if requested_policy_index is None:
@@ -366,7 +389,7 @@ class ContainerController(object):
             metadata = {}
             metadata.update(
                 (key, (value, req_timestamp.internal))
-                for key, value in req.headers.iteritems()
+                for key, value in req.headers.items()
                 if key.lower() in self.save_headers or
                 is_sys_or_user_meta('container', key))
             if 'X-Container-Sync-To' in metadata:
@@ -375,13 +398,19 @@ class ContainerController(object):
                         broker.metadata['X-Container-Sync-To'][0]:
                     broker.set_x_container_sync_points(-1, -1)
             broker.update_metadata(metadata, validate_metadata=True)
+            if metadata:
+                self._update_sync_store(broker, 'PUT')
             resp = self.account_update(req, account, container, broker)
             if resp:
                 return resp
             if created:
-                return HTTPCreated(request=req)
+                return HTTPCreated(request=req,
+                                   headers={'x-backend-storage-policy-index':
+                                            broker.storage_policy_index})
             else:
-                return HTTPAccepted(request=req)
+                return HTTPAccepted(request=req,
+                                    headers={'x-backend-storage-policy-index':
+                                             broker.storage_policy_index})
 
     @public
     @timing_stats(sample_rate=0.1)
@@ -401,11 +430,13 @@ class ContainerController(object):
             return HTTPNotFound(request=req, headers=headers)
         headers.update(
             (key, value)
-            for key, (value, timestamp) in broker.metadata.iteritems()
+            for key, (value, timestamp) in broker.metadata.items()
             if value != '' and (key.lower() in self.save_headers or
                                 is_sys_or_user_meta('container', key)))
         headers['Content-Type'] = out_content_type
-        return HTTPNoContent(request=req, headers=headers, charset='utf-8')
+        resp = HTTPNoContent(request=req, headers=headers, charset='utf-8')
+        resp.last_modified = math.ceil(float(headers['X-PUT-Timestamp']))
+        return resp
 
     def update_data_record(self, record):
         """
@@ -443,6 +474,7 @@ class ContainerController(object):
         end_marker = get_param(req, 'end_marker')
         limit = constraints.CONTAINER_LISTING_LIMIT
         given_limit = get_param(req, 'limit')
+        reverse = config_true_value(get_param(req, 'reverse'))
         if given_limit and given_limit.isdigit():
             limit = int(given_limit)
             if limit > constraints.CONTAINER_LISTING_LIMIT:
@@ -462,13 +494,13 @@ class ContainerController(object):
             return HTTPNotFound(request=req, headers=resp_headers)
         container_list = broker.list_objects_iter(
             limit, marker, end_marker, prefix, delimiter, path,
-            storage_policy_index=info['storage_policy_index'])
+            storage_policy_index=info['storage_policy_index'], reverse=reverse)
         return self.create_listing(req, out_content_type, info, resp_headers,
                                    broker.metadata, container_list, container)
 
     def create_listing(self, req, out_content_type, info, resp_headers,
                        metadata, container_list, container):
-        for key, (value, timestamp) in metadata.iteritems():
+        for key, (value, timestamp) in metadata.items():
             if value and (key.lower() in self.save_headers or
                           is_sys_or_user_meta('container', key)):
                 resp_headers[key] = value
@@ -501,6 +533,7 @@ class ContainerController(object):
             if not container_list:
                 return HTTPNoContent(request=req, headers=resp_headers)
             ret.body = '\n'.join(rec[0] for rec in container_list) + '\n'
+        ret.last_modified = math.ceil(float(resp_headers['X-PUT-Timestamp']))
         return ret
 
     @public
@@ -539,10 +572,11 @@ class ContainerController(object):
         broker = self._get_container_broker(drive, part, account, container)
         if broker.is_deleted():
             return HTTPNotFound(request=req)
+        broker.update_put_timestamp(req_timestamp.internal)
         metadata = {}
         metadata.update(
             (key, (value, req_timestamp.internal))
-            for key, value in req.headers.iteritems()
+            for key, value in req.headers.items()
             if key.lower() in self.save_headers or
             is_sys_or_user_meta('container', key))
         if metadata:
@@ -552,6 +586,7 @@ class ContainerController(object):
                         broker.metadata['X-Container-Sync-To'][0]:
                     broker.set_x_container_sync_points(-1, -1)
             broker.update_metadata(metadata, validate_metadata=True)
+            self._update_sync_store(broker, 'POST')
         return HTTPNoContent(request=req)
 
     def __call__(self, env, start_response):
@@ -563,17 +598,10 @@ class ContainerController(object):
         else:
             try:
                 # disallow methods which have not been marked 'public'
-                try:
-                    method = getattr(self, req.method)
-                    getattr(method, 'publicly_accessible')
-                    replication_method = getattr(method, 'replication', False)
-                    if (self.replication_server is not None and
-                            self.replication_server != replication_method):
-                        raise AttributeError('Not allowed method.')
-                except AttributeError:
+                if req.method not in self.allowed_methods:
                     res = HTTPMethodNotAllowed()
                 else:
-                    res = method(req)
+                    res = getattr(self, req.method)(req)
             except HTTPException as error_response:
                 res = error_response
             except (Exception, Timeout):
