@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import print_function
 import logging
 import os
 import signal
@@ -30,7 +31,7 @@ from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout
 from swift.common.ring import Ring
 from swift.common.utils import get_logger, config_true_value, ismount, \
-    dump_recon_cache, quorum_size, Timestamp
+    dump_recon_cache, majority_size, Timestamp
 from swift.common.daemon import Daemon
 from swift.common.http import is_success, HTTP_INTERNAL_SERVER_ERROR
 
@@ -48,7 +49,7 @@ class ContainerUpdater(Daemon):
         self.account_ring = None
         self.concurrency = int(conf.get('concurrency', 4))
         self.slowdown = float(conf.get('slowdown', 0.01))
-        self.node_timeout = int(conf.get('node_timeout', 3))
+        self.node_timeout = float(conf.get('node_timeout', 3))
         self.conn_timeout = float(conf.get('conn_timeout', 0.5))
         self.no_changes = 0
         self.successes = 0
@@ -70,6 +71,14 @@ class ContainerUpdater(Daemon):
             self.account_ring = Ring(self.swift_dir, ring_name='account')
         return self.account_ring
 
+    def _listdir(self, path):
+        try:
+            return os.listdir(path)
+        except OSError as e:
+            self.logger.error(_('ERROR:  Failed to get paths to drive '
+                                'partitions: %s') % e)
+            return []
+
     def get_paths(self):
         """
         Get paths to all of the partitions on each drive to be processed.
@@ -77,15 +86,15 @@ class ContainerUpdater(Daemon):
         :returns: a list of paths
         """
         paths = []
-        for device in os.listdir(self.devices):
+        for device in self._listdir(self.devices):
             dev_path = os.path.join(self.devices, device)
             if self.mount_check and not ismount(dev_path):
-                self.logger.warn(_('%s is not mounted'), device)
+                self.logger.warning(_('%s is not mounted'), device)
                 continue
             con_path = os.path.join(dev_path, DATADIR)
             if not os.path.exists(con_path):
                 continue
-            for partition in os.listdir(con_path):
+            for partition in self._listdir(con_path):
                 paths.append(os.path.join(con_path, partition))
         shuffle(paths)
         return paths
@@ -105,7 +114,7 @@ class ContainerUpdater(Daemon):
 
     def run_forever(self, *args, **kwargs):
         """
-        Run the updator continuously.
+        Run the updater continuously.
         """
         time.sleep(random() * self.interval)
         while True:
@@ -113,7 +122,7 @@ class ContainerUpdater(Daemon):
             begin = time.time()
             now = time.time()
             expired_suppressions = \
-                [a for a, u in self.account_suppressions.iteritems()
+                [a for a, u in self.account_suppressions.items()
                  if u < now]
             for account in expired_suppressions:
                 del self.account_suppressions[account]
@@ -134,7 +143,8 @@ class ContainerUpdater(Daemon):
                     pid2filename[pid] = tmpfilename
                 else:
                     signal.signal(signal.SIGTERM, signal.SIG_DFL)
-                    patcher.monkey_patch(all=False, socket=True)
+                    patcher.monkey_patch(all=False, socket=True, select=True,
+                                         thread=True)
                     self.no_changes = 0
                     self.successes = 0
                     self.failures = 0
@@ -168,7 +178,7 @@ class ContainerUpdater(Daemon):
         """
         Run the updater once.
         """
-        patcher.monkey_patch(all=False, socket=True)
+        patcher.monkey_patch(all=False, socket=True, select=True, thread=True)
         self.logger.info(_('Begin container update single threaded sweep'))
         begin = time.time()
         self.no_changes = 0
@@ -228,7 +238,7 @@ class ContainerUpdater(Daemon):
             for event in events:
                 if is_success(event.wait()):
                     successes += 1
-            if successes >= quorum_size(len(events)):
+            if successes >= majority_size(len(events)):
                 self.logger.increment('successes')
                 self.successes += 1
                 self.logger.debug(
@@ -246,8 +256,8 @@ class ContainerUpdater(Daemon):
                 self.account_suppressions[info['account']] = until = \
                     time.time() + self.account_suppression_time
                 if self.new_account_suppressions:
-                    print >>self.new_account_suppressions, \
-                        info['account'], until
+                    print(info['account'], until,
+                          file=self.new_account_suppressions)
             # Only track timing data for attempted updates:
             self.logger.timing_since('timing', start_time)
         else:
