@@ -29,6 +29,8 @@ import sqlite3
 import itertools
 from contextlib import contextmanager
 import random
+import mock
+import base64
 
 from swift.account.backend import AccountBroker
 from swift.common.utils import Timestamp
@@ -36,7 +38,7 @@ from test.unit import patch_policies, with_tempdir, make_timestamp_iter
 from swift.common.db import DatabaseConnectionError
 from swift.common.storage_policy import StoragePolicy, POLICIES
 
-from test.unit.common.test_db import TestExampleBroker
+from test.unit.common import test_db
 
 
 @patch_policies
@@ -65,6 +67,13 @@ class TestAccountBroker(unittest.TestCase):
             curs.execute('SELECT 1')
             self.assertEqual(curs.fetchall()[0][0], 1)
 
+    def test_initialize_fail(self):
+        broker = AccountBroker(':memory:')
+        with self.assertRaises(ValueError) as cm:
+            broker.initialize(Timestamp('1').internal)
+        self.assertEqual(str(cm.exception), 'Attempting to create a new'
+                         ' database with no account set')
+
     def test_exception(self):
         # Test AccountBroker throwing a conn away after exception
         first_conn = None
@@ -416,20 +425,48 @@ class TestAccountBroker(unittest.TestCase):
         self.assertEqual(listing[0][0], '0-0100')
         self.assertEqual(listing[-1][0], '0-0109')
 
+        listing = broker.list_containers_iter(10, '', None, '0-00', '-',
+                                              reverse=True)
+        self.assertEqual(len(listing), 10)
+        self.assertEqual(listing[0][0], '0-0099')
+        self.assertEqual(listing[-1][0], '0-0090')
+
         listing = broker.list_containers_iter(10, '', None, '0-', '-')
         self.assertEqual(len(listing), 10)
         self.assertEqual(listing[0][0], '0-0000')
         self.assertEqual(listing[-1][0], '0-0009')
 
+        listing = broker.list_containers_iter(10, '', None, '0-', '-',
+                                              reverse=True)
+        self.assertEqual(len(listing), 10)
+        self.assertEqual(listing[0][0], '0-0124')
+        self.assertEqual(listing[-1][0], '0-0115')
+
         listing = broker.list_containers_iter(10, '', None, '', '-')
         self.assertEqual(len(listing), 4)
         self.assertEqual([row[0] for row in listing],
                          ['0-', '1-', '2-', '3-'])
 
+        listing = broker.list_containers_iter(10, '', None, '', '-',
+                                              reverse=True)
+        self.assertEqual(len(listing), 4)
+        self.assertEqual([row[0] for row in listing],
+                         ['3-', '2-', '1-', '0-'])
+
         listing = broker.list_containers_iter(10, '2-', None, None, '-')
         self.assertEqual(len(listing), 1)
         self.assertEqual([row[0] for row in listing], ['3-'])
 
+        listing = broker.list_containers_iter(10, '2-', None, None, '-',
+                                              reverse=True)
+        self.assertEqual(len(listing), 2)
+        self.assertEqual([row[0] for row in listing], ['1-', '0-'])
+
+        listing = broker.list_containers_iter(10, '2.', None, None, '-',
+                                              reverse=True)
+        self.assertEqual(len(listing), 3)
+        self.assertEqual([row[0] for row in listing], ['2-', '1-', '0-'])
+
         listing = broker.list_containers_iter(10, '', None, '2', '-')
         self.assertEqual(len(listing), 1)
         self.assertEqual([row[0] for row in listing], ['2-'])
@@ -469,6 +506,168 @@ class TestAccountBroker(unittest.TestCase):
         self.assertEqual([row[0] for row in listing],
                          ['3-0049-', '3-0049-0049'])
 
+    def test_list_objects_iter_order_and_reverse(self):
+        # Test ContainerBroker.list_objects_iter
+        broker = AccountBroker(':memory:', account='a')
+        broker.initialize(Timestamp('1').internal, 0)
+
+        broker.put_container(
+            'c1', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+        broker.put_container(
+            'c10', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+        broker.put_container(
+            'C1', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+        broker.put_container(
+            'c2', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+        broker.put_container(
+            'c3', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+        broker.put_container(
+            'C4', Timestamp(0).internal, 0, 0, 0, POLICIES.default.idx)
+
+        listing = broker.list_containers_iter(100, None, None, '', '',
+                                              reverse=False)
+        self.assertEqual([row[0] for row in listing],
+                         ['C1', 'C4', 'c1', 'c10', 'c2', 'c3'])
+        listing = broker.list_containers_iter(100, None, None, '', '',
+                                              reverse=True)
+        self.assertEqual([row[0] for row in listing],
+                         ['c3', 'c2', 'c10', 'c1', 'C4', 'C1'])
+        listing = broker.list_containers_iter(2, None, None, '', '',
+                                              reverse=True)
+        self.assertEqual([row[0] for row in listing],
+                         ['c3', 'c2'])
+        listing = broker.list_containers_iter(100, 'c2', 'C4', '', '',
+                                              reverse=True)
+        self.assertEqual([row[0] for row in listing],
+                         ['c10', 'c1'])
+
+    def test_reverse_prefix_delim(self):
+        expectations = [
+            {
+                'containers': [
+                    'topdir1-subdir1,0-c1',
+                    'topdir1-subdir1,1-c1',
+                    'topdir1-subdir1-c1',
+                ],
+                'params': {
+                    'prefix': 'topdir1-',
+                    'delimiter': '-',
+                },
+                'expected': [
+                    'topdir1-subdir1,0-',
+                    'topdir1-subdir1,1-',
+                    'topdir1-subdir1-',
+                ],
+            },
+            {
+                'containers': [
+                    'topdir1-subdir1,0-c1',
+                    'topdir1-subdir1,1-c1',
+                    'topdir1-subdir1-c1',
+                    'topdir1-subdir1.',
+                    'topdir1-subdir1.-c1',
+                ],
+                'params': {
+                    'prefix': 'topdir1-',
+                    'delimiter': '-',
+                },
+                'expected': [
+                    'topdir1-subdir1,0-',
+                    'topdir1-subdir1,1-',
+                    'topdir1-subdir1-',
+                    'topdir1-subdir1.',
+                    'topdir1-subdir1.-',
+                ],
+            },
+            {
+                'containers': [
+                    'topdir1-subdir1-c1',
+                    'topdir1-subdir1,0-c1',
+                    'topdir1-subdir1,1-c1',
+                ],
+                'params': {
+                    'prefix': 'topdir1-',
+                    'delimiter': '-',
+                    'reverse': True,
+                },
+                'expected': [
+                    'topdir1-subdir1-',
+                    'topdir1-subdir1,1-',
+                    'topdir1-subdir1,0-',
+                ],
+            },
+            {
+                'containers': [
+                    'topdir1-subdir1.-c1',
+                    'topdir1-subdir1.',
+                    'topdir1-subdir1-c1',
+                    'topdir1-subdir1-',
+                    'topdir1-subdir1,',
+                    'topdir1-subdir1,0-c1',
+                    'topdir1-subdir1,1-c1',
+                ],
+                'params': {
+                    'prefix': 'topdir1-',
+                    'delimiter': '-',
+                    'reverse': True,
+                },
+                'expected': [
+                    'topdir1-subdir1.-',
+                    'topdir1-subdir1.',
+                    'topdir1-subdir1-',
+                    'topdir1-subdir1,1-',
+                    'topdir1-subdir1,0-',
+                    'topdir1-subdir1,',
+                ],
+            },
+            {
+                'containers': [
+                    '1',
+                    '2',
+                    '3:1',
+                    '3:2:1',
+                    '3:2:2',
+                    '3:3',
+                    '4',
+                ],
+                'params': {
+                    'prefix': '3:',
+                    'delimiter': ':',
+                    'reverse': True,
+                },
+                'expected': [
+                    '3:3',
+                    '3:2:',
+                    '3:1',
+                ],
+            },
+        ]
+        ts = make_timestamp_iter()
+        default_listing_params = {
+            'limit': 10000,
+            'marker': '',
+            'end_marker': None,
+            'prefix': None,
+            'delimiter': None,
+        }
+        failures = []
+        for expected in expectations:
+            broker = AccountBroker(':memory:', account='a')
+            broker.initialize(next(ts).internal, 0)
+            for name in expected['containers']:
+                broker.put_container(name, next(ts).internal, 0, 0, 0,
+                                     POLICIES.default.idx)
+            params = default_listing_params.copy()
+            params.update(expected['params'])
+            listing = list(c[0] for c in broker.list_containers_iter(**params))
+            if listing != expected['expected']:
+                expected['listing'] = listing
+                failures.append(
+                    "With containers %(containers)r, the params %(params)r "
+                    "produced %(listing)r instead of %(expected)r" % expected)
+        self.assertFalse(failures, "Found the following failures:\n%s" %
+                         '\n'.join(failures))
+
     def test_double_check_trailing_delimiter(self):
         # Test AccountBroker.list_containers_iter for an
         # account that has an odd container with a trailing delimiter
@@ -595,33 +794,79 @@ class TestAccountBroker(unittest.TestCase):
         self.assertEqual(items_by_name['b']['object_count'], 0)
         self.assertEqual(items_by_name['b']['bytes_used'], 0)
 
-    def test_load_old_pending_puts(self):
+    @with_tempdir
+    def test_load_old_pending_puts(self, tempdir):
         # pending puts from pre-storage-policy account brokers won't contain
         # the storage policy index
-        tempdir = mkdtemp()
         broker_path = os.path.join(tempdir, 'test-load-old.db')
-        try:
-            broker = AccountBroker(broker_path, account='real')
-            broker.initialize(Timestamp(1).internal)
-            with open(broker_path + '.pending', 'a+b') as pending:
-                pending.write(':')
-                pending.write(pickle.dumps(
-                    # name, put_timestamp, delete_timestamp, object_count,
-                    # bytes_used, deleted
-                    ('oldcon', Timestamp(200).internal,
-                     Timestamp(0).internal,
-                     896, 9216695, 0)).encode('base64'))
+        broker = AccountBroker(broker_path, account='real')
+        broker.initialize(Timestamp(1).internal)
+        with open(broker.pending_file, 'a+b') as pending:
+            pending.write(b':')
+            pending.write(base64.b64encode(pickle.dumps(
+                # name, put_timestamp, delete_timestamp, object_count,
+                # bytes_used, deleted
+                ('oldcon', Timestamp(200).internal,
+                 Timestamp(0).internal,
+                 896, 9216695, 0))))
 
-            broker._commit_puts()
-            with broker.get() as conn:
-                results = list(conn.execute('''
-                    SELECT name, storage_policy_index FROM container
-                '''))
-            self.assertEqual(len(results), 1)
-            self.assertEqual(dict(results[0]),
-                             {'name': 'oldcon', 'storage_policy_index': 0})
-        finally:
-            rmtree(tempdir)
+        broker._commit_puts()
+        with broker.get() as conn:
+            results = list(conn.execute('''
+                SELECT name, storage_policy_index FROM container
+            '''))
+        self.assertEqual(len(results), 1)
+        self.assertEqual(dict(results[0]),
+                         {'name': 'oldcon', 'storage_policy_index': 0})
+
+    @with_tempdir
+    def test_get_info_stale_read_ok(self, tempdir):
+        # test getting a stale read from the db
+        broker_path = os.path.join(tempdir, 'test-load-old.db')
+
+        def mock_commit_puts():
+            raise sqlite3.OperationalError('unable to open database file')
+
+        broker = AccountBroker(broker_path, account='real',
+                               stale_reads_ok=True)
+        broker.initialize(Timestamp(1).internal)
+        with open(broker.pending_file, 'a+b') as pending:
+            pending.write(b':')
+            pending.write(base64.b64encode(pickle.dumps(
+                # name, put_timestamp, delete_timestamp, object_count,
+                # bytes_used, deleted
+                ('oldcon', Timestamp(200).internal,
+                 Timestamp(0).internal,
+                 896, 9216695, 0))))
+
+        broker._commit_puts = mock_commit_puts
+        broker.get_info()
+
+    @with_tempdir
+    def test_get_info_no_stale_reads(self, tempdir):
+        broker_path = os.path.join(tempdir, 'test-load-old.db')
+
+        def mock_commit_puts():
+            raise sqlite3.OperationalError('unable to open database file')
+
+        broker = AccountBroker(broker_path, account='real',
+                               stale_reads_ok=False)
+        broker.initialize(Timestamp(1).internal)
+        with open(broker.pending_file, 'a+b') as pending:
+            pending.write(b':')
+            pending.write(base64.b64encode(pickle.dumps(
+                # name, put_timestamp, delete_timestamp, object_count,
+                # bytes_used, deleted
+                ('oldcon', Timestamp(200).internal,
+                 Timestamp(0).internal,
+                 896, 9216695, 0))))
+
+        broker._commit_puts = mock_commit_puts
+
+        with self.assertRaises(sqlite3.OperationalError) as exc_context:
+            broker.get_info()
+        self.assertIn('unable to open database file',
+                      str(exc_context.exception))
 
     @patch_policies([StoragePolicy(0, 'zero', False),
                      StoragePolicy(1, 'one', True),
@@ -781,7 +1026,7 @@ def premetadata_create_account_stat_table(self, conn, put_timestamp):
               put_timestamp))
 
 
-class TestCommonAccountBroker(TestExampleBroker):
+class TestCommonAccountBroker(test_db.TestExampleBroker):
 
     broker_class = AccountBroker
 
@@ -1487,3 +1732,15 @@ class TestAccountBrokerBeforePerPolicyContainerTrack(
         self.assertEqual(len(policy_info), 2)
         for policy_stat in policy_info.values():
             self.assertEqual(policy_stat['container_count'], 1)
+
+    def test_migrate_add_storage_policy_index_fail(self):
+        broker = AccountBroker(':memory:', account='a')
+        broker.initialize(Timestamp('1').internal)
+        with mock.patch.object(
+                broker, 'create_policy_stat_table',
+                side_effect=sqlite3.OperationalError('foobar')):
+            with broker.get() as conn:
+                self.assertRaisesRegexp(
+                    sqlite3.OperationalError, '.*foobar.*',
+                    broker._migrate_add_storage_policy_index,
+                    conn=conn)
