@@ -22,7 +22,8 @@ import unittest
 
 from logging import DEBUG
 from mock import patch, call, DEFAULT
-from contextlib import nested
+import six
+import eventlet
 
 from swift.account import reaper
 from swift.account.backend import DATADIR
@@ -99,16 +100,24 @@ class FakeRing(object):
     def __init__(self):
         self.nodes = [{'id': '1',
                        'ip': '10.10.10.1',
-                       'port': 6002,
-                       'device': None},
+                       'port': 6202,
+                       'device': 'sda1'},
                       {'id': '2',
                        'ip': '10.10.10.2',
-                       'port': 6002,
-                       'device': None},
+                       'port': 6202,
+                       'device': 'sda1'},
                       {'id': '3',
                        'ip': '10.10.10.3',
-                       'port': 6002,
+                       'port': 6202,
                        'device': None},
+                      {'id': '4',
+                       'ip': '10.10.10.1',
+                       'port': 6202,
+                       'device': 'sda2'},
+                      {'id': '5',
+                       'ip': '10.10.10.1',
+                       'port': 6202,
+                       'device': 'sda3'},
                       ]
 
     def get_nodes(self, *args, **kwargs):
@@ -124,6 +133,12 @@ acc_nodes = [{'device': 'sda1',
              {'device': 'sda1',
               'ip': '',
               'port': ''},
+             {'device': 'sda1',
+              'ip': '',
+              'port': ''},
+             {'device': 'sda1',
+              'ip': '',
+              'port': ''},
              {'device': 'sda1',
               'ip': '',
               'port': ''}]
@@ -134,6 +149,12 @@ cont_nodes = [{'device': 'sda1',
               {'device': 'sda1',
                'ip': '',
                'port': ''},
+              {'device': 'sda1',
+               'ip': '',
+               'port': ''},
+              {'device': 'sda1',
+               'ip': '',
+               'port': ''},
               {'device': 'sda1',
                'ip': '',
                'port': ''}]
@@ -162,6 +183,8 @@ class TestReaper(unittest.TestCase):
         if self.amount_fail < self.max_fail:
             self.amount_fail += 1
             raise self.myexp
+        if self.reap_obj_timeout:
+            raise eventlet.Timeout()
 
     def fake_direct_delete_container(self, *args, **kwargs):
         if self.amount_delete_fail < self.max_delete_fail:
@@ -171,9 +194,11 @@ class TestReaper(unittest.TestCase):
     def fake_direct_get_container(self, *args, **kwargs):
         if self.get_fail:
             raise self.myexp
+        if self.timeout:
+            raise eventlet.Timeout()
         objects = [{'name': 'o1'},
                    {'name': 'o2'},
-                   {'name': unicode('o3')},
+                   {'name': six.text_type('o3')},
                    {'name': ''}]
         return None, objects
 
@@ -184,11 +209,11 @@ class TestReaper(unittest.TestCase):
         if self.reap_obj_fail:
             raise Exception
 
-    def prepare_data_dir(self, ts=False):
+    def prepare_data_dir(self, ts=False, device='sda1'):
         devices_path = tempfile.mkdtemp()
         # will be deleted by teardown
         self.to_delete.append(devices_path)
-        path = os.path.join(devices_path, 'sda1', DATADIR)
+        path = os.path.join(devices_path, device, DATADIR)
         os.makedirs(path)
         path = os.path.join(path, '100',
                             'a86', 'a8c682d2472e1720f2d81ff8993aba6')
@@ -207,13 +232,6 @@ class TestReaper(unittest.TestCase):
             myips = ['10.10.10.1']
 
         r = reaper.AccountReaper(conf)
-        r.stats_return_codes = {}
-        r.stats_containers_deleted = 0
-        r.stats_containers_remaining = 0
-        r.stats_containers_possibly_remaining = 0
-        r.stats_objects_deleted = 0
-        r.stats_objects_remaining = 0
-        r.stats_objects_possibly_remaining = 0
         r.myips = myips
         if fakelogger:
             r.logger = unit.debug_logger('test-reaper')
@@ -225,6 +243,11 @@ class TestReaper(unittest.TestCase):
     def fake_account_ring(self):
         return FakeRing()
 
+    def test_creation(self):
+        # later config should be extended to assert more config options
+        r = reaper.AccountReaper({'node_timeout': '3.5'})
+        self.assertEqual(r.node_timeout, 3.5)
+
     def test_delay_reaping_conf_default(self):
         r = reaper.AccountReaper({})
         self.assertEqual(r.delay_reaping, 0)
@@ -274,6 +297,33 @@ class TestReaper(unittest.TestCase):
         finally:
             reaper.time = time_orig
 
+    def test_reset_stats(self):
+        conf = {}
+        r = reaper.AccountReaper(conf)
+        self.assertDictEqual(r.stats_return_codes, {})
+        self.assertEqual(r.stats_containers_deleted, 0)
+        self.assertEqual(r.stats_containers_remaining, 0)
+        self.assertEqual(r.stats_containers_possibly_remaining, 0)
+        self.assertEqual(r.stats_objects_deleted, 0)
+        self.assertEqual(r.stats_objects_remaining, 0)
+        self.assertEqual(r.stats_objects_possibly_remaining, 0)
+        # also make sure reset actually resets values
+        r.stats_return_codes = {"hello": "swift"}
+        r.stats_containers_deleted = random.randint(1, 100)
+        r.stats_containers_remaining = random.randint(1, 100)
+        r.stats_containers_possibly_remaining = random.randint(1, 100)
+        r.stats_objects_deleted = random.randint(1, 100)
+        r.stats_objects_remaining = random.randint(1, 100)
+        r.stats_objects_possibly_remaining = random.randint(1, 100)
+        r.reset_stats()
+        self.assertDictEqual(r.stats_return_codes, {})
+        self.assertEqual(r.stats_containers_deleted, 0)
+        self.assertEqual(r.stats_containers_remaining, 0)
+        self.assertEqual(r.stats_containers_possibly_remaining, 0)
+        self.assertEqual(r.stats_objects_deleted, 0)
+        self.assertEqual(r.stats_objects_remaining, 0)
+        self.assertEqual(r.stats_objects_possibly_remaining, 0)
+
     def test_reap_object(self):
         conf = {
             'mount_check': 'false',
@@ -314,6 +364,7 @@ class TestReaper(unittest.TestCase):
         r = self.init_reaper({}, fakelogger=True)
         self.amount_fail = 0
         self.max_fail = 1
+        self.reap_obj_timeout = False
         policy = random.choice(list(POLICIES))
         with patch('swift.account.reaper.direct_delete_object',
                    self.fake_direct_delete_object):
@@ -341,6 +392,23 @@ class TestReaper(unittest.TestCase):
                          policy.object_ring.replicas - 2)
         self.assertEqual(r.stats_objects_remaining, 1)
         self.assertEqual(r.stats_objects_possibly_remaining, 1)
+        self.assertEqual(r.stats_return_codes[2],
+                         policy.object_ring.replicas - 1)
+        self.assertEqual(r.stats_return_codes[4], 1)
+
+    def test_reap_object_timeout(self):
+        r = self.init_reaper({}, fakelogger=True)
+        self.amount_fail = 1
+        self.max_fail = 0
+        self.reap_obj_timeout = True
+        with patch('swift.account.reaper.direct_delete_object',
+                   self.fake_direct_delete_object):
+            r.reap_object('a', 'c', 'partition', cont_nodes, 'o', 1)
+        self.assertEqual(r.stats_objects_deleted, 0)
+        self.assertEqual(r.stats_objects_remaining, 4)
+        self.assertEqual(r.stats_objects_possibly_remaining, 0)
+        self.assertTrue(r.logger.get_lines_for_level(
+            'error')[-1].startswith('Timeout Exception'))
 
     def test_reap_object_non_exist_policy_index(self):
         r = self.init_reaper({}, fakelogger=True)
@@ -414,15 +482,14 @@ class TestReaper(unittest.TestCase):
         self.reap_obj_fail = False
         self.amount_delete_fail = 0
         self.max_delete_fail = 0
-        ctx = [patch('swift.account.reaper.direct_get_container',
-                     self.fake_direct_get_container),
-               patch('swift.account.reaper.direct_delete_container',
-                     self.fake_direct_delete_container),
-               patch('swift.account.reaper.AccountReaper.get_container_ring',
-                     self.fake_container_ring),
-               patch('swift.account.reaper.AccountReaper.reap_object',
-                     self.fake_reap_object)]
-        with nested(*ctx):
+        with patch('swift.account.reaper.direct_get_container',
+                   self.fake_direct_get_container), \
+                patch('swift.account.reaper.direct_delete_container',
+                      self.fake_direct_delete_container), \
+                patch('swift.account.reaper.AccountReaper.get_container_ring',
+                      self.fake_container_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_object',
+                      self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 1)
         self.assertEqual(r.stats_containers_deleted, 1)
@@ -430,41 +497,60 @@ class TestReaper(unittest.TestCase):
     def test_reap_container_partial_fail(self):
         r = self.init_reaper({}, fakelogger=True)
         self.get_fail = False
+        self.timeout = False
         self.reap_obj_fail = False
         self.amount_delete_fail = 0
-        self.max_delete_fail = 2
-        ctx = [patch('swift.account.reaper.direct_get_container',
-                     self.fake_direct_get_container),
-               patch('swift.account.reaper.direct_delete_container',
-                     self.fake_direct_delete_container),
-               patch('swift.account.reaper.AccountReaper.get_container_ring',
-                     self.fake_container_ring),
-               patch('swift.account.reaper.AccountReaper.reap_object',
-                     self.fake_reap_object)]
-        with nested(*ctx):
+        self.max_delete_fail = 4
+        with patch('swift.account.reaper.direct_get_container',
+                   self.fake_direct_get_container), \
+                patch('swift.account.reaper.direct_delete_container',
+                      self.fake_direct_delete_container), \
+                patch('swift.account.reaper.AccountReaper.get_container_ring',
+                      self.fake_container_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_object',
+                      self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
-        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 2)
+        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 4)
         self.assertEqual(r.stats_containers_possibly_remaining, 1)
 
     def test_reap_container_full_fail(self):
         r = self.init_reaper({}, fakelogger=True)
         self.get_fail = False
+        self.timeout = False
         self.reap_obj_fail = False
         self.amount_delete_fail = 0
-        self.max_delete_fail = 3
-        ctx = [patch('swift.account.reaper.direct_get_container',
-                     self.fake_direct_get_container),
-               patch('swift.account.reaper.direct_delete_container',
-                     self.fake_direct_delete_container),
-               patch('swift.account.reaper.AccountReaper.get_container_ring',
-                     self.fake_container_ring),
-               patch('swift.account.reaper.AccountReaper.reap_object',
-                     self.fake_reap_object)]
-        with nested(*ctx):
+        self.max_delete_fail = 5
+        with patch('swift.account.reaper.direct_get_container',
+                   self.fake_direct_get_container), \
+                patch('swift.account.reaper.direct_delete_container',
+                      self.fake_direct_delete_container), \
+                patch('swift.account.reaper.AccountReaper.get_container_ring',
+                      self.fake_container_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_object',
+                      self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
-        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 3)
+        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 5)
         self.assertEqual(r.stats_containers_remaining, 1)
 
+    def test_reap_container_get_object_timeout(self):
+        r = self.init_reaper({}, fakelogger=True)
+        self.get_fail = False
+        self.timeout = True
+        self.reap_obj_fail = False
+        self.amount_delete_fail = 0
+        self.max_delete_fail = 0
+        with patch('swift.account.reaper.direct_get_container',
+                   self.fake_direct_get_container), \
+                patch('swift.account.reaper.direct_delete_container',
+                      self.fake_direct_delete_container), \
+                patch('swift.account.reaper.AccountReaper.get_container_ring',
+                      self.fake_container_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_object',
+                      self.fake_reap_object):
+            r.reap_container('a', 'partition', acc_nodes, 'c')
+        self.assertTrue(r.logger.get_lines_for_level(
+            'error')[-1].startswith('Timeout Exception'))
+
     @patch('swift.account.reaper.Ring',
            lambda *args, **kwargs: unit.FakeRing())
     def test_reap_container_non_exist_policy_index(self):
@@ -498,6 +584,8 @@ class TestReaper(unittest.TestCase):
         self.r.stats_objects_remaining = 1
         self.r.stats_containers_possibly_remaining = 1
         self.r.stats_objects_possibly_remaining = 1
+        self.r.stats_return_codes[2] = \
+            self.r.stats_return_codes.get(2, 0) + 1
 
     def test_reap_account(self):
         containers = ('c1', 'c2', 'c3', '')
@@ -516,7 +604,7 @@ class TestReaper(unittest.TestCase):
                                    container_shard=container_shard))
         self.assertEqual(self.called_amount, 4)
         info_lines = r.logger.get_lines_for_level('info')
-        self.assertEqual(len(info_lines), 6)
+        self.assertEqual(len(info_lines), 10)
         for start_line, stat_line in zip(*[iter(info_lines)] * 2):
             self.assertEqual(start_line, 'Beginning pass on account a')
             self.assertTrue(stat_line.find('1 containers deleted'))
@@ -525,17 +613,26 @@ class TestReaper(unittest.TestCase):
             self.assertTrue(stat_line.find('1 objects remaining'))
             self.assertTrue(stat_line.find('1 containers possibly remaining'))
             self.assertTrue(stat_line.find('1 objects possibly remaining'))
+            self.assertTrue(stat_line.find('return codes: 2 2xxs'))
+
+    @patch('swift.account.reaper.Ring',
+           lambda *args, **kwargs: unit.FakeRing())
+    def test_basic_reap_account(self):
+        self.r = reaper.AccountReaper({})
+        self.r.account_ring = None
+        self.r.get_account_ring()
+        self.assertEqual(self.r.account_ring.replica_count, 3)
+        self.assertEqual(len(self.r.account_ring.devs), 3)
 
     def test_reap_account_no_container(self):
         broker = FakeAccountBroker(tuple())
         self.r = r = self.init_reaper({}, fakelogger=True)
         self.called_amount = 0
         r.start_time = time.time()
-        ctx = [patch('swift.account.reaper.AccountReaper.reap_container',
-                     self.fake_reap_container),
-               patch('swift.account.reaper.AccountReaper.get_account_ring',
-                     self.fake_account_ring)]
-        with nested(*ctx):
+        with patch('swift.account.reaper.AccountReaper.reap_container',
+                   self.fake_reap_container), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring):
             nodes = r.get_account_ring().get_part_nodes()
             self.assertTrue(r.reap_account(broker, 'partition', nodes))
         self.assertTrue(r.logger.get_lines_for_level(
@@ -547,13 +644,12 @@ class TestReaper(unittest.TestCase):
         self.called_amount = 0
         conf = {'devices': devices}
         r = self.init_reaper(conf)
-        ctx = [patch('swift.account.reaper.AccountBroker',
-                     FakeAccountBroker),
-               patch('swift.account.reaper.AccountReaper.get_account_ring',
-                     self.fake_account_ring),
-               patch('swift.account.reaper.AccountReaper.reap_account',
-                     self.fake_reap_account)]
-        with nested(*ctx):
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_account',
+                      self.fake_reap_account):
             r.reap_device('sda1')
         self.assertEqual(self.called_amount, 1)
 
@@ -562,13 +658,12 @@ class TestReaper(unittest.TestCase):
         self.called_amount = 0
         conf = {'devices': devices}
         r = self.init_reaper(conf=conf)
-        ctx = [patch('swift.account.reaper.AccountBroker',
-                     FakeAccountBroker),
-               patch('swift.account.reaper.AccountReaper.get_account_ring',
-                     self.fake_account_ring),
-               patch('swift.account.reaper.AccountReaper.reap_account',
-                     self.fake_reap_account)]
-        with nested(*ctx):
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_account',
+                      self.fake_reap_account):
             r.reap_device('sda1')
         self.assertEqual(self.called_amount, 0)
 
@@ -577,13 +672,12 @@ class TestReaper(unittest.TestCase):
         self.called_amount = 0
         conf = {'devices': devices}
         r = self.init_reaper(conf, myips=['10.10.1.2'])
-        ctx = [patch('swift.account.reaper.AccountBroker',
-                     FakeAccountBroker),
-               patch('swift.account.reaper.AccountReaper.get_account_ring',
-                     self.fake_account_ring),
-               patch('swift.account.reaper.AccountReaper.reap_account',
-                     self.fake_reap_account)]
-        with nested(*ctx):
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_account',
+                      self.fake_reap_account):
             r.reap_device('sda1')
         self.assertEqual(self.called_amount, 0)
 
@@ -606,6 +700,42 @@ class TestReaper(unittest.TestCase):
         # 10.10.10.2 is second node from ring
         self.assertEqual(container_shard_used[0], 1)
 
+    def test_reap_device_with_sharding_and_various_devices(self):
+        devices = self.prepare_data_dir(device='sda2')
+        conf = {'devices': devices}
+        r = self.init_reaper(conf)
+        container_shard_used = [-1]
+
+        def fake_reap_account(*args, **kwargs):
+            container_shard_used[0] = kwargs.get('container_shard')
+
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_account',
+                      fake_reap_account):
+            r.reap_device('sda2')
+
+        # 10.10.10.2 is second node from ring
+        self.assertEqual(container_shard_used[0], 3)
+
+        devices = self.prepare_data_dir(device='sda3')
+        conf = {'devices': devices}
+        r = self.init_reaper(conf)
+        container_shard_used = [-1]
+
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch('swift.account.reaper.AccountReaper.get_account_ring',
+                      self.fake_account_ring), \
+                patch('swift.account.reaper.AccountReaper.reap_account',
+                      fake_reap_account):
+            r.reap_device('sda3')
+
+        # 10.10.10.2 is second node from ring
+        self.assertEqual(container_shard_used[0], 4)
+
     def test_reap_account_with_sharding(self):
         devices = self.prepare_data_dir()
         self.called_amount = 0
@@ -626,28 +756,39 @@ class TestReaper(unittest.TestCase):
                                 account_nodes, container):
             container_reaped[0] += 1
 
-        ctx = [patch('swift.account.reaper.AccountBroker',
-                     FakeAccountBroker),
-               patch('swift.account.reaper.AccountBroker.list_containers_iter',
-                     fake_list_containers_iter),
-               patch('swift.account.reaper.AccountReaper.reap_container',
-                     fake_reap_container), ]
         fake_ring = FakeRing()
-        with nested(*ctx):
-            fake_broker = FakeAccountBroker(['c', 'd', 'e'])
-            r.reap_account(fake_broker, 10, fake_ring.nodes, 0)
-            self.assertEqual(container_reaped[0], 1)
+        with patch('swift.account.reaper.AccountBroker',
+                   FakeAccountBroker), \
+                patch(
+                    'swift.account.reaper.AccountBroker.list_containers_iter',
+                    fake_list_containers_iter), \
+                patch('swift.account.reaper.AccountReaper.reap_container',
+                      fake_reap_container):
 
-            fake_broker = FakeAccountBroker(['c', 'd', 'e'])
+            fake_broker = FakeAccountBroker(['c', 'd', 'e', 'f', 'g'])
+            r.reap_account(fake_broker, 10, fake_ring.nodes, 0)
+            self.assertEqual(container_reaped[0], 0)
+
+            fake_broker = FakeAccountBroker(['c', 'd', 'e', 'f', 'g'])
             container_reaped[0] = 0
             r.reap_account(fake_broker, 10, fake_ring.nodes, 1)
-            self.assertEqual(container_reaped[0], 2)
+            self.assertEqual(container_reaped[0], 1)
 
             container_reaped[0] = 0
-            fake_broker = FakeAccountBroker(['c', 'd', 'e'])
+            fake_broker = FakeAccountBroker(['c', 'd', 'e', 'f', 'g'])
             r.reap_account(fake_broker, 10, fake_ring.nodes, 2)
             self.assertEqual(container_reaped[0], 0)
 
+            container_reaped[0] = 0
+            fake_broker = FakeAccountBroker(['c', 'd', 'e', 'f', 'g'])
+            r.reap_account(fake_broker, 10, fake_ring.nodes, 3)
+            self.assertEqual(container_reaped[0], 3)
+
+            container_reaped[0] = 0
+            fake_broker = FakeAccountBroker(['c', 'd', 'e', 'f', 'g'])
+            r.reap_account(fake_broker, 10, fake_ring.nodes, 4)
+            self.assertEqual(container_reaped[0], 1)
+
     def test_run_once(self):
         def prepare_data_dir():
             devices_path = tempfile.mkdtemp()
@@ -676,6 +817,13 @@ class TestReaper(unittest.TestCase):
                 r.run_once()
         self.assertFalse(foo.called)
 
+        with patch('swift.account.reaper.AccountReaper.reap_device') as foo:
+            r.logger = unit.debug_logger('test-reaper')
+            r.devices = 'thisdeviceisbad'
+            r.run_once()
+        self.assertTrue(r.logger.get_lines_for_level(
+            'error')[-1].startswith('Exception in top-level account reaper'))
+
     def test_run_forever(self):
         def fake_sleep(val):
             self.val = val
