@@ -1,4 +1,4 @@
-#-*- coding:utf-8 -*-
+# coding: utf-8
 # Copyright (c) 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import contextlib
 import hashlib
 import json
 import mock
@@ -25,7 +24,9 @@ import time
 import unittest
 
 from swift.common import exceptions, swob
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.middleware import dlo
+from swift.common.utils import closing_if_possible
 from test.unit.common.middleware.helpers import FakeSwift
 
 
@@ -54,8 +55,10 @@ class DloTestCase(unittest.TestCase):
         body = ''
         caught_exc = None
         try:
-            for chunk in body_iter:
-                body += chunk
+            # appease the close-checker
+            with closing_if_possible(body_iter):
+                for chunk in body_iter:
+                    body += chunk
         except Exception as exc:
             if expect_exception:
                 caught_exc = exc
@@ -73,7 +76,7 @@ class DloTestCase(unittest.TestCase):
             # don't slow down tests with rate limiting
             'rate_limit_after_segment': '1000000',
         })(self.app)
-
+        self.dlo.logger = self.app.logger
         self.app.register(
             'GET', '/v1/AUTH_test/c/seg_01',
             swob.HTTPOk, {'Content-Length': '5', 'Etag': md5hex("aaaaa")},
@@ -246,7 +249,7 @@ class TestDloHeadManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'HEAD'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"], expected_etag)
         self.assertEqual(headers["Content-Length"], "25")
 
@@ -255,7 +258,7 @@ class TestDloHeadManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'HEAD'})
         with mock.patch(LIMIT, 3):
             status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         # etag is manifest's etag
         self.assertEqual(headers["Etag"], "etag-manyseg")
@@ -265,7 +268,7 @@ class TestDloHeadManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest-no-segments',
                                  environ={'REQUEST_METHOD': 'HEAD'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"], '"%s"' % md5hex(""))
         self.assertEqual(headers["Content-Length"], "0")
 
@@ -279,6 +282,9 @@ class TestDloHeadManifest(DloTestCase):
 
 
 class TestDloGetManifest(DloTestCase):
+    def tearDown(self):
+        self.assertEqual(self.app.unclosed_requests, {})
+
     def test_get_manifest(self):
         expected_etag = '"%s"' % md5hex(
             md5hex("aaaaa") + md5hex("bbbbb") + md5hex("ccccc") +
@@ -286,7 +292,7 @@ class TestDloGetManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"], expected_etag)
         self.assertEqual(headers["Content-Length"], "25")
         self.assertEqual(body, 'aaaaabbbbbcccccdddddeeeee')
@@ -331,7 +337,7 @@ class TestDloGetManifest(DloTestCase):
             environ={'REQUEST_METHOD': 'GET',
                      'QUERY_STRING': 'multipart-manifest=get'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"], "manifest-etag")
         self.assertEqual(body, "manifest-contents")
 
@@ -349,7 +355,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=8-17'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "10")
         self.assertEqual(body, "bbcccccddd")
@@ -363,7 +369,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=10-19'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "10")
         self.assertEqual(body, "cccccddddd")
@@ -373,7 +379,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=0-0'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "1")
         self.assertEqual(body, "a")
@@ -383,7 +389,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=24-24'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "1")
         self.assertEqual(body, "e")
@@ -393,7 +399,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=18-30'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "7")
         self.assertEqual(headers["Content-Range"], "bytes 18-24/25")
@@ -412,7 +418,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'Range': 'bytes=3-12'})
         with mock.patch(LIMIT, 3):
             status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "10")
         # The /15 here indicates that this is a 15-byte object. DLO can't tell
@@ -443,7 +449,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'Range': 'bytes=10-22'})
         with mock.patch(LIMIT, 3):
             status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "200 OK")
         # this requires multiple pages of container listing, so we can't send
         # a Content-Length header
@@ -455,7 +461,7 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'},
                                  headers={'Range': 'bytes=-40'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "206 Partial Content")
         self.assertEqual(headers["Content-Length"], "25")
         self.assertEqual(body, "aaaaabbbbbcccccdddddeeeee")
@@ -466,7 +472,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'Range': 'bytes=-5'})
         with mock.patch(LIMIT, 3):
             status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "200 OK")
         self.assertEqual(headers.get("Content-Length"), None)
         self.assertEqual(headers.get("Content-Range"), None)
@@ -480,7 +486,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'Range': 'bytes=5-9,15-19'})
         with mock.patch(LIMIT, 3):
             status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(status, "200 OK")
         self.assertEqual(headers.get("Content-Length"), None)
         self.assertEqual(headers.get("Content-Range"), None)
@@ -495,7 +501,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'If-Match': manifest_etag})
 
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '200 OK')
         self.assertEqual(headers['Content-Length'], '25')
@@ -507,7 +513,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'If-Match': 'not it'})
 
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '412 Precondition Failed')
         self.assertEqual(headers['Content-Length'], '0')
@@ -522,7 +528,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'If-None-Match': manifest_etag})
 
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '304 Not Modified')
         self.assertEqual(headers['Content-Length'], '0')
@@ -534,7 +540,7 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'If-None-Match': 'not it'})
 
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '200 OK')
         self.assertEqual(headers['Content-Length'], '25')
@@ -562,12 +568,12 @@ class TestDloGetManifest(DloTestCase):
 
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
-        status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
-        self.assertTrue(isinstance(exc, exceptions.SegmentError))
-
-        self.assertEqual(status, "200 OK")
-        self.assertEqual(body, '')  # error right away -> no body bytes sent
+        status, headers, body = self.call_dlo(req)
+        self.assertEqual(status, "409 Conflict")
+        err_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(err_lines), 1)
+        self.assertTrue(err_lines[0].startswith(
+            'ERROR: An error occurred while retrieving segments'))
 
     def test_error_fetching_second_segment(self):
         self.app.register(
@@ -577,11 +583,15 @@ class TestDloGetManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
         status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertTrue(isinstance(exc, exceptions.SegmentError))
         self.assertEqual(status, "200 OK")
         self.assertEqual(''.join(body), "aaaaa")  # first segment made it out
+        err_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(err_lines), 1)
+        self.assertTrue(err_lines[0].startswith(
+            'ERROR: An error occurred while retrieving segments'))
 
     def test_error_listing_container_first_listing_request(self):
         self.app.register(
@@ -619,14 +629,14 @@ class TestDloGetManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
         status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertTrue(isinstance(exc, exceptions.SegmentError))
         self.assertEqual(status, "200 OK")
         self.assertEqual(''.join(body), "aaaaabbWRONGbb")  # stop after error
 
     def test_etag_comparison_ignores_quotes(self):
-        # a little future-proofing here in case we ever fix this
+        # a little future-proofing here in case we ever fix this in swob
         self.app.register(
             'HEAD', '/v1/AUTH_test/mani/festo',
             swob.HTTPOk, {'Content-Length': '0', 'Etag': 'blah',
@@ -644,7 +654,7 @@ class TestDloGetManifest(DloTestCase):
         req = swob.Request.blank('/v1/AUTH_test/mani/festo',
                                  environ={'REQUEST_METHOD': 'HEAD'})
         status, headers, body = self.call_dlo(req)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"],
                          '"' + hashlib.md5("abcdef").hexdigest() + '"')
 
@@ -691,12 +701,11 @@ class TestDloGetManifest(DloTestCase):
             '/v1/AUTH_test/mancon/manifest',
             environ={'REQUEST_METHOD': 'GET'})
 
-        with contextlib.nested(
-                mock.patch('swift.common.request_helpers.time.time',
-                           mock_time),
+        with mock.patch('swift.common.request_helpers.time.time',
+                        mock_time), \
                 mock.patch('swift.common.request_helpers.is_success',
-                           mock_is_success),
-                mock.patch.object(dlo, 'is_success', mock_is_success)):
+                           mock_is_success), \
+                mock.patch.object(dlo, 'is_success', mock_is_success):
             status, headers, body, exc = self.call_dlo(
                 req, expect_exception=True)
 
@@ -721,7 +730,7 @@ class TestDloGetManifest(DloTestCase):
             '/v1/AUTH_test/mancon/manifest',
             environ={'REQUEST_METHOD': 'GET'})
         status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '200 OK')  # sanity check
         self.assertEqual(headers.get('Content-Length'), '25')  # sanity check
@@ -754,7 +763,7 @@ class TestDloGetManifest(DloTestCase):
             '/v1/AUTH_test/mancon/manifest',
             environ={'REQUEST_METHOD': 'GET'})
         status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '200 OK')  # sanity check
         self.assertEqual(headers.get('Content-Length'), '25')  # sanity check
@@ -773,7 +782,7 @@ class TestDloGetManifest(DloTestCase):
             environ={'REQUEST_METHOD': 'GET'},
             headers={'Range': 'bytes=0-14'})
         status, headers, body, exc = self.call_dlo(req, expect_exception=True)
-        headers = swob.HeaderKeyDict(headers)
+        headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, '206 Partial Content')  # sanity check
         self.assertEqual(headers.get('Content-Length'), '15')  # sanity check
@@ -783,7 +792,7 @@ class TestDloGetManifest(DloTestCase):
     def test_get_with_auth_overridden(self):
         auth_got_called = [0]
 
-        def my_auth():
+        def my_auth(req):
             auth_got_called[0] += 1
             return None
 
@@ -794,107 +803,6 @@ class TestDloGetManifest(DloTestCase):
         self.assertTrue(auth_got_called[0] > 1)
 
 
-def fake_start_response(*args, **kwargs):
-    pass
-
-
-class TestDloCopyHook(DloTestCase):
-    def setUp(self):
-        super(TestDloCopyHook, self).setUp()
-
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/o1', swob.HTTPOk,
-            {'Content-Length': '10', 'Etag': 'o1-etag'},
-            "aaaaaaaaaa")
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/o2', swob.HTTPOk,
-            {'Content-Length': '10', 'Etag': 'o2-etag'},
-            "bbbbbbbbbb")
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/man',
-            swob.HTTPOk, {'X-Object-Manifest': 'c/o'},
-            "manifest-contents")
-
-        lm = '2013-11-22T02:42:13.781760'
-        ct = 'application/octet-stream'
-        segs = [{"hash": "o1-etag", "bytes": 10, "name": "o1",
-                 "last_modified": lm, "content_type": ct},
-                {"hash": "o2-etag", "bytes": 5, "name": "o2",
-                 "last_modified": lm, "content_type": ct}]
-
-        self.app.register(
-            'GET', '/v1/AUTH_test/c?format=json&prefix=o',
-            swob.HTTPOk, {'Content-Type': 'application/json; charset=utf-8'},
-            json.dumps(segs))
-
-        copy_hook = [None]
-
-        # slip this guy in there to pull out the hook
-        def extract_copy_hook(env, sr):
-            copy_hook[0] = env.get('swift.copy_hook')
-            return self.app(env, sr)
-
-        self.dlo = dlo.filter_factory({})(extract_copy_hook)
-
-        req = swob.Request.blank('/v1/AUTH_test/c/o1',
-                                 environ={'REQUEST_METHOD': 'GET'})
-        self.dlo(req.environ, fake_start_response)
-        self.copy_hook = copy_hook[0]
-
-        self.assertTrue(self.copy_hook is not None)  # sanity check
-
-    def test_copy_hook_passthrough(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(request=source_req, status=200)
-
-        # no X-Object-Manifest header, so do nothing
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is source_resp)
-
-    def test_copy_hook_manifest(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(
-            request=source_req, status=200,
-            headers={"X-Object-Manifest": "c/o"},
-            app_iter=["manifest"])
-
-        # it's a manifest, so copy the segments to make a normal object
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is not source_resp)
-        self.assertEqual(modified_resp.etag,
-                         hashlib.md5("o1-etago2-etag").hexdigest())
-        self.assertEqual(sink_req.headers.get('X-Object-Manifest'), None)
-
-    def test_copy_hook_manifest_with_multipart_manifest_get(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET',
-                     'QUERY_STRING': 'multipart-manifest=get'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(
-            request=source_req, status=200,
-            headers={"X-Object-Manifest": "c/o"},
-            app_iter=["manifest"])
-
-        # make sure the sink request (the backend PUT) gets X-Object-Manifest
-        # on it, but that's all
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is source_resp)
-        self.assertEqual(sink_req.headers.get('X-Object-Manifest'), 'c/o')
-
-
 class TestDloConfiguration(unittest.TestCase):
     """
     For backwards compatibility, we will read a couple of values out of the
