@@ -16,21 +16,25 @@
 import json
 import unittest
 import os
-import urllib
 from contextlib import contextmanager
 from hashlib import md5
 import time
+import pickle
 
 import mock
 import six
+from six.moves import urllib
 
 from swift.common import direct_client
+from swift.common.direct_client import DirectClientException
 from swift.common.exceptions import ClientException
-from swift.common.utils import Timestamp
-from swift.common.swob import HeaderKeyDict, RESPONSE_REASONS
+from swift.common.header_key_dict import HeaderKeyDict
+from swift.common.utils import Timestamp, quote
+from swift.common.swob import RESPONSE_REASONS
 from swift.common.storage_policy import POLICIES
+from six.moves.http_client import HTTPException
 
-from test.unit import patch_policies
+from test.unit import patch_policies, debug_logger
 
 
 class FakeConn(object):
@@ -45,7 +49,6 @@ class FakeConn(object):
         self.resp_headers = HeaderKeyDict()
         if headers:
             self.resp_headers.update(headers)
-        self.with_exc = False
         self.etag = None
 
     def _update_raw_call_args(self, *args, **kwargs):
@@ -58,8 +61,8 @@ class FakeConn(object):
     def getresponse(self):
         if self.etag:
             self.resp_headers['etag'] = str(self.etag.hexdigest())
-        if self.with_exc:
-            raise Exception('test')
+        if isinstance(self.status, Exception):
+            raise self.status
         return self
 
     def getheader(self, header, default=None):
@@ -68,8 +71,13 @@ class FakeConn(object):
     def getheaders(self):
         return self.resp_headers.items()
 
-    def read(self):
-        return self.body
+    def read(self, amt=None):
+        if isinstance(self.body, six.StringIO):
+            return self.body.read(amt)
+        elif amt is None:
+            return self.body
+        else:
+            return Exception('Not a StringIO entry')
 
     def send(self, data):
         if not self.etag:
@@ -91,19 +99,20 @@ def mocked_http_conn(*args, **kwargs):
 class TestDirectClient(unittest.TestCase):
 
     def setUp(self):
-        self.node = {'ip': '1.2.3.4', 'port': '6000', 'device': 'sda'}
+        self.node = {'ip': '1.2.3.4', 'port': '6200', 'device': 'sda',
+                     'replication_ip': '1.2.3.5', 'replication_port': '7000'}
         self.part = '0'
 
         self.account = u'\u062a account'
         self.container = u'\u062a container'
         self.obj = u'\u062a obj/name'
-        self.account_path = '/sda/0/%s' % urllib.quote(
+        self.account_path = '/sda/0/%s' % urllib.parse.quote(
             self.account.encode('utf-8'))
         self.container_path = '/sda/0/%s/%s' % tuple(
-            urllib.quote(p.encode('utf-8')) for p in (
+            urllib.parse.quote(p.encode('utf-8')) for p in (
                 self.account, self.container))
         self.obj_path = '/sda/0/%s/%s/%s' % tuple(
-            urllib.quote(p.encode('utf-8')) for p in (
+            urllib.parse.quote(p.encode('utf-8')) for p in (
                 self.account, self.container, self.obj))
         self.user_agent = 'direct-client %s' % os.getpid()
 
@@ -151,30 +160,47 @@ class TestDirectClient(unittest.TestCase):
                 self.assertEqual(expected_header_count, len(headers))
 
     def test_direct_get_account(self):
-        stub_headers = HeaderKeyDict({
-            'X-Account-Container-Count': '1',
-            'X-Account-Object-Count': '1',
-            'X-Account-Bytes-Used': '1',
-            'X-Timestamp': '1234567890',
-            'X-PUT-Timestamp': '1234567890'})
+        def do_test(req_params):
+            stub_headers = HeaderKeyDict({
+                'X-Account-Container-Count': '1',
+                'X-Account-Object-Count': '1',
+                'X-Account-Bytes-Used': '1',
+                'X-Timestamp': '1234567890',
+                'X-PUT-Timestamp': '1234567890'})
 
-        body = '[{"count": 1, "bytes": 20971520, "name": "c1"}]'
+            body = '[{"count": 1, "bytes": 20971520, "name": "c1"}]'
 
-        with mocked_http_conn(200, stub_headers, body) as conn:
-            resp_headers, resp = direct_client.direct_get_account(
-                self.node, self.part, self.account, marker='marker',
-                prefix='prefix', delimiter='delimiter', limit=1000)
-            self.assertEqual(conn.method, 'GET')
-            self.assertEqual(conn.path, self.account_path)
+            with mocked_http_conn(200, stub_headers, body) as conn:
+                resp_headers, resp = direct_client.direct_get_account(
+                    self.node, self.part, self.account, **req_params)
+            try:
+                self.assertEqual(conn.method, 'GET')
+                self.assertEqual(conn.path, self.account_path)
+                self.assertEqual(conn.req_headers['user-agent'],
+                                 self.user_agent)
+                self.assertEqual(resp_headers, stub_headers)
+                self.assertEqual(json.loads(body), resp)
+                self.assertIn('format=json', conn.query_string)
+                for k, v in req_params.items():
+                    if v is None:
+                        self.assertNotIn('&%s' % k, conn.query_string)
+                    else:
+                        self.assertIn('&%s=%s' % (k, v), conn.query_string)
 
-        self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
-        self.assertEqual(resp_headers, stub_headers)
-        self.assertEqual(json.loads(body), resp)
-        self.assertTrue('marker=marker' in conn.query_string)
-        self.assertTrue('delimiter=delimiter' in conn.query_string)
-        self.assertTrue('limit=1000' in conn.query_string)
-        self.assertTrue('prefix=prefix' in conn.query_string)
-        self.assertTrue('format=json' in conn.query_string)
+            except AssertionError as err:
+                self.fail('Failed with params %s: %s' % (req_params, err))
+
+        test_params = (dict(marker=marker, prefix=prefix, delimiter=delimiter,
+                       limit=limit, end_marker=end_marker, reverse=reverse)
+                       for marker in (None, 'my-marker')
+                       for prefix in (None, 'my-prefix')
+                       for delimiter in (None, 'my-delimiter')
+                       for limit in (None, 1000)
+                       for end_marker in (None, 'my-endmarker')
+                       for reverse in (None, 'on'))
+
+        for params in test_params:
+            do_test(params)
 
     def test_direct_client_exception(self):
         stub_headers = {'X-Trans-Id': 'txb5f59485c578460f8be9e-0053478d09'}
@@ -209,7 +235,7 @@ class TestDirectClient(unittest.TestCase):
             'X-Account-Object-Count': '1',
             'X-Account-Bytes-Used': '1',
             'X-Timestamp': '1234567890',
-            'X-PUT-Timestamp': '1234567890'}
+            'X-Put-Timestamp': '1234567890'}
         with mocked_http_conn(204, headers) as conn:
             resp_headers, resp = direct_client.direct_get_account(
                 self.node, self.part, self.account)
@@ -217,7 +243,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.account_path)
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
-        self.assertEqual(resp_headers, resp_headers)
+        self.assertDictEqual(resp_headers, headers)
         self.assertEqual([], resp)
 
     def test_direct_get_account_error(self):
@@ -235,14 +261,13 @@ class TestDirectClient(unittest.TestCase):
         self.assertTrue('GET' in str(err))
 
     def test_direct_delete_account(self):
-        node = {'ip': '1.2.3.4', 'port': '6000', 'device': 'sda'}
         part = '0'
         account = 'a'
 
         mock_path = 'swift.common.bufferedhttp.http_connect_raw'
         with mock.patch(mock_path) as fake_connect:
             fake_connect.return_value.getresponse.return_value.status = 200
-            direct_client.direct_delete_account(node, part, account)
+            direct_client.direct_delete_account(self.node, part, account)
             args, kwargs = fake_connect.call_args
             method = args[2]
             self.assertEqual('DELETE', method)
@@ -251,6 +276,19 @@ class TestDirectClient(unittest.TestCase):
             headers = args[4]
             self.assertTrue('X-Timestamp' in headers)
 
+    def test_direct_delete_account_failure(self):
+        part = '0'
+        account = 'a'
+
+        with mocked_http_conn(500) as conn:
+            try:
+                direct_client.direct_delete_account(self.node, part, account)
+            except ClientException as err:
+                pass
+            self.assertEqual('DELETE', conn.method)
+            self.assertEqual('/sda/0/a', conn.path)
+            self.assertEqual(err.http_status, 500)
+
     def test_direct_head_container(self):
         headers = HeaderKeyDict(key='value')
 
@@ -305,24 +343,43 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(err.http_headers, headers)
 
     def test_direct_get_container(self):
-        headers = HeaderKeyDict({'key': 'value'})
-        body = '[{"hash": "8f4e3", "last_modified": "317260", "bytes": 209}]'
+        def do_test(req_params):
+            headers = HeaderKeyDict({'key': 'value'})
+            body = ('[{"hash": "8f4e3", "last_modified": "317260", '
+                    '"bytes": 209}]')
 
-        with mocked_http_conn(200, headers, body) as conn:
-            resp_headers, resp = direct_client.direct_get_container(
-                self.node, self.part, self.account, self.container,
-                marker='marker', prefix='prefix', delimiter='delimiter',
-                limit=1000)
+            with mocked_http_conn(200, headers, body) as conn:
+                resp_headers, resp = direct_client.direct_get_container(
+                    self.node, self.part, self.account, self.container,
+                    **req_params)
 
-        self.assertEqual(conn.req_headers['user-agent'],
-                         'direct-client %s' % os.getpid())
-        self.assertEqual(headers, resp_headers)
-        self.assertEqual(json.loads(body), resp)
-        self.assertTrue('marker=marker' in conn.query_string)
-        self.assertTrue('delimiter=delimiter' in conn.query_string)
-        self.assertTrue('limit=1000' in conn.query_string)
-        self.assertTrue('prefix=prefix' in conn.query_string)
-        self.assertTrue('format=json' in conn.query_string)
+            try:
+                self.assertEqual(conn.method, 'GET')
+                self.assertEqual(conn.path, self.container_path)
+                self.assertEqual(conn.req_headers['user-agent'],
+                                 self.user_agent)
+                self.assertEqual(headers, resp_headers)
+                self.assertEqual(json.loads(body), resp)
+                self.assertIn('format=json', conn.query_string)
+                for k, v in req_params.items():
+                    if v is None:
+                        self.assertNotIn('&%s' % k, conn.query_string)
+                    else:
+                        self.assertIn('&%s=%s' % (k, v), conn.query_string)
+            except AssertionError as err:
+                self.fail('Failed with params %s: %s' % (req_params, err))
+
+        test_params = (dict(marker=marker, prefix=prefix, delimiter=delimiter,
+                       limit=limit, end_marker=end_marker, reverse=reverse)
+                       for marker in (None, 'my-marker')
+                       for prefix in (None, 'my-prefix')
+                       for delimiter in (None, 'my-delimiter')
+                       for limit in (None, 1000)
+                       for end_marker in (None, 'my-endmarker')
+                       for reverse in (None, 'on'))
+
+        for params in test_params:
+            do_test(params)
 
     def test_direct_get_container_no_content_does_not_decode_body(self):
         headers = {}
@@ -488,7 +545,7 @@ class TestDirectClient(unittest.TestCase):
                 self.node, self.part, self.account, self.container, self.obj)
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(obj_body, contents)
+        self.assertEqual(obj_body, contents.getvalue())
 
     def test_direct_get_object_error(self):
         with mocked_http_conn(500) as conn:
@@ -506,6 +563,24 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(err.http_status, 500)
         self.assertTrue('GET' in str(err))
 
+    def test_direct_get_object_chunks(self):
+        contents = six.StringIO('123456')
+        downloaded = b''
+
+        with mocked_http_conn(200, body=contents) as conn:
+            resp_header, obj_body = direct_client.direct_get_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                resp_chunk_size=2)
+            while obj_body:
+                try:
+                    chunk = obj_body.next()
+                except StopIteration:
+                    break
+                downloaded += chunk
+            self.assertEqual('GET', conn.method)
+            self.assertEqual(self.obj_path, conn.path)
+        self.assertEqual('123456', downloaded)
+
     def test_direct_post_object(self):
         headers = {'Key': 'value'}
 
@@ -579,6 +654,40 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(err.http_status, 503)
         self.assertTrue('DELETE' in str(err))
 
+    def test_direct_get_suffix_hashes(self):
+        data = {'a83': 'c130a2c17ed45102aada0f4eee69494ff'}
+        body = pickle.dumps(data)
+        with mocked_http_conn(200, {}, body) as conn:
+            resp = direct_client.direct_get_suffix_hashes(self.node,
+                                                          self.part, ['a83'])
+            self.assertEqual(conn.method, 'REPLICATE')
+            self.assertEqual(conn.path, '/sda/0/a83')
+            self.assertEqual(conn.host, '1.2.3.5')
+            self.assertEqual(conn.port, '7000')
+            self.assertEqual(data, resp)
+
+    def _test_direct_get_suffix_hashes_fail(self, status_code):
+        with mocked_http_conn(status_code):
+            with self.assertRaises(DirectClientException) as cm:
+                direct_client.direct_get_suffix_hashes(
+                    self.node, self.part, ['a83', 'b52'])
+        self.assertIn('REPLICATE', cm.exception.message)
+        self.assertIn(quote('/%s/%s/a83-b52'
+                            % (self.node['device'], self.part)),
+                      cm.exception.message)
+        self.assertIn(self.node['replication_ip'], cm.exception.message)
+        self.assertIn(self.node['replication_port'], cm.exception.message)
+        self.assertEqual(self.node['replication_ip'], cm.exception.http_host)
+        self.assertEqual(self.node['replication_port'], cm.exception.http_port)
+        self.assertEqual(self.node['device'], cm.exception.http_device)
+        self.assertEqual(status_code, cm.exception.http_status)
+
+    def test_direct_get_suffix_hashes_503(self):
+        self._test_direct_get_suffix_hashes_fail(503)
+
+    def test_direct_get_suffix_hashes_507(self):
+        self._test_direct_get_suffix_hashes_fail(507)
+
     def test_direct_put_object_with_content_length(self):
         contents = six.StringIO('123456')
 
@@ -617,6 +726,32 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
         self.assertEqual(md5('6\r\n123456\r\n0\r\n\r\n').hexdigest(), resp)
 
+    def test_direct_put_object_args(self):
+        # One test to cover all missing checks
+        contents = ""
+        with mocked_http_conn(200) as conn:
+            resp = direct_client.direct_put_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                contents, etag="testing-etag", content_type='Text')
+            self.assertEqual('PUT', conn.method)
+            self.assertEqual(self.obj_path, conn.path)
+            self.assertEqual(conn.req_headers['Content-Length'], '0')
+            self.assertEqual(conn.req_headers['Content-Type'], 'Text')
+        self.assertEqual(md5('0\r\n\r\n').hexdigest(), resp)
+
+    def test_direct_put_object_header_content_length(self):
+        contents = six.StringIO('123456')
+        stub_headers = HeaderKeyDict({
+            'Content-Length': '6'})
+
+        with mocked_http_conn(200) as conn:
+            resp = direct_client.direct_put_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                contents, headers=stub_headers)
+            self.assertEqual('PUT', conn.method)
+            self.assertEqual(conn.req_headers['Content-length'], '6')
+        self.assertEqual(md5('123456').hexdigest(), resp)
+
     def test_retry(self):
         headers = HeaderKeyDict({'key': 'value'})
 
@@ -630,6 +765,54 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(headers, resp)
         self.assertEqual(attempts, 1)
 
+    def test_retry_client_exception(self):
+        logger = debug_logger('direct-client-test')
+
+        with mock.patch('swift.common.direct_client.sleep') as mock_sleep, \
+                mocked_http_conn(500) as conn:
+            with self.assertRaises(direct_client.ClientException) as err_ctx:
+                direct_client.retry(direct_client.direct_delete_object,
+                                    self.node, self.part,
+                                    self.account, self.container, self.obj,
+                                    retries=2, error_log=logger.error)
+        self.assertEqual('DELETE', conn.method)
+        self.assertEqual(err_ctx.exception.http_status, 500)
+        self.assertIn('DELETE', err_ctx.exception.message)
+        self.assertIn(quote('/%s/%s/%s/%s/%s'
+                            % (self.node['device'], self.part, self.account,
+                               self.container, self.obj)),
+                      err_ctx.exception.message)
+        self.assertIn(self.node['ip'], err_ctx.exception.message)
+        self.assertIn(self.node['port'], err_ctx.exception.message)
+        self.assertEqual(self.node['ip'], err_ctx.exception.http_host)
+        self.assertEqual(self.node['port'], err_ctx.exception.http_port)
+        self.assertEqual(self.node['device'], err_ctx.exception.http_device)
+        self.assertEqual(500, err_ctx.exception.http_status)
+        self.assertEqual([mock.call(1), mock.call(2)],
+                         mock_sleep.call_args_list)
+        error_lines = logger.get_lines_for_level('error')
+        self.assertEqual(3, len(error_lines))
+        for line in error_lines:
+            self.assertIn('500 Internal Error', line)
+
+    def test_retry_http_exception(self):
+        logger = debug_logger('direct-client-test')
+
+        with mock.patch('swift.common.direct_client.sleep') as mock_sleep, \
+                mocked_http_conn(HTTPException('Kaboom!')) as conn:
+            with self.assertRaises(HTTPException) as err_ctx:
+                direct_client.retry(direct_client.direct_delete_object,
+                                    self.node, self.part,
+                                    self.account, self.container, self.obj,
+                                    retries=2, error_log=logger.error)
+        self.assertEqual('DELETE', conn.method)
+        self.assertEqual('Kaboom!', str(err_ctx.exception))
+        self.assertEqual([mock.call(1), mock.call(2)],
+                         mock_sleep.call_args_list)
+        error_lines = logger.get_lines_for_level('error')
+        self.assertEqual(3, len(error_lines))
+        for line in error_lines:
+            self.assertIn('Kaboom!', line)
 
 if __name__ == '__main__':
     unittest.main()
