@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import cPickle as pickle
+import six.moves.cPickle as pickle
 import mock
 import os
 import unittest
@@ -23,20 +23,23 @@ from contextlib import closing
 from gzip import GzipFile
 from tempfile import mkdtemp
 from shutil import rmtree
+from test import listen_zero
+from test.unit import FakeLogger
+from test.unit import debug_logger, patch_policies, mocked_http_conn
 from time import time
 from distutils.dir_util import mkpath
 
-from eventlet import spawn, Timeout, listen
+from eventlet import spawn, Timeout
+from six.moves import range
 
 from swift.obj import updater as object_updater
 from swift.obj.diskfile import (ASYNCDIR_BASE, get_async_dir, DiskFileManager,
                                 get_tmp_dir)
 from swift.common.ring import RingData
 from swift.common import utils
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.utils import hash_path, normalize_timestamp, mkdirs, \
     write_pickle
-from swift.common import swob
-from test.unit import debug_logger, patch_policies, mocked_http_conn
 from swift.common.storage_policy import StoragePolicy, POLICIES
 
 
@@ -69,7 +72,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.sda1 = os.path.join(self.devices_dir, 'sda1')
         os.mkdir(self.sda1)
         for policy in POLICIES:
-            os.mkdir(os.path.join(self.sda1, get_tmp_dir(int(policy))))
+            os.mkdir(os.path.join(self.sda1, get_tmp_dir(policy)))
         self.logger = debug_logger()
 
     def tearDown(self):
@@ -82,14 +85,47 @@ class TestObjectUpdater(unittest.TestCase):
             'swift_dir': self.testdir,
             'interval': '1',
             'concurrency': '2',
-            'node_timeout': '5'})
-        self.assert_(hasattr(cu, 'logger'))
-        self.assert_(cu.logger is not None)
-        self.assertEquals(cu.devices, self.devices_dir)
-        self.assertEquals(cu.interval, 1)
-        self.assertEquals(cu.concurrency, 2)
-        self.assertEquals(cu.node_timeout, 5)
-        self.assert_(cu.get_container_ring() is not None)
+            'node_timeout': '5.5'})
+        self.assertTrue(hasattr(cu, 'logger'))
+        self.assertTrue(cu.logger is not None)
+        self.assertEqual(cu.devices, self.devices_dir)
+        self.assertEqual(cu.interval, 1)
+        self.assertEqual(cu.concurrency, 2)
+        self.assertEqual(cu.node_timeout, 5.5)
+        self.assertTrue(cu.get_container_ring() is not None)
+
+    @mock.patch('os.listdir')
+    def test_listdir_with_exception(self, mock_listdir):
+        e = OSError('permission_denied')
+        mock_listdir.side_effect = e
+        # setup updater
+        conf = {
+            'devices': self.devices_dir,
+            'mount_check': 'false',
+            'swift_dir': self.testdir,
+        }
+        daemon = object_updater.ObjectUpdater(conf)
+        daemon.logger = FakeLogger()
+        paths = daemon._listdir('foo/bar')
+        self.assertEqual([], paths)
+        log_lines = daemon.logger.get_lines_for_level('error')
+        msg = ('ERROR: Unable to access foo/bar: permission_denied')
+        self.assertEqual(log_lines[0], msg)
+
+    @mock.patch('os.listdir', return_value=['foo', 'bar'])
+    def test_listdir_without_exception(self, mock_listdir):
+        # setup updater
+        conf = {
+            'devices': self.devices_dir,
+            'mount_check': 'false',
+            'swift_dir': self.testdir,
+        }
+        daemon = object_updater.ObjectUpdater(conf)
+        daemon.logger = FakeLogger()
+        path = daemon._listdir('foo/bar/')
+        log_lines = daemon.logger.get_lines_for_level('error')
+        self.assertEqual(len(log_lines), 0)
+        self.assertEqual(path, ['foo', 'bar'])
 
     def test_object_sweep(self):
         def check_with_idx(index, warn, should_skip):
@@ -123,7 +159,7 @@ class TestObjectUpdater(unittest.TestCase):
             }
 
             expected = set()
-            for o, timestamps in objects.iteritems():
+            for o, timestamps in objects.items():
                 ohash = hash_path('account', 'container', o)
                 for t in timestamps:
                     o_path = os.path.join(prefix_dir, ohash + '-' +
@@ -135,8 +171,8 @@ class TestObjectUpdater(unittest.TestCase):
             seen = set()
 
             class MockObjectUpdater(object_updater.ObjectUpdater):
-                def process_object_update(self, update_path, device, idx):
-                    seen.add((update_path, idx))
+                def process_object_update(self, update_path, device, policy):
+                    seen.add((update_path, int(policy)))
                     os.unlink(update_path)
 
             cu = MockObjectUpdater({
@@ -148,15 +184,16 @@ class TestObjectUpdater(unittest.TestCase):
                 'node_timeout': '5'})
             cu.logger = mock_logger = mock.MagicMock()
             cu.object_sweep(self.sda1)
-            self.assertEquals(mock_logger.warn.call_count, warn)
-            self.assert_(os.path.exists(os.path.join(self.sda1, 'not_a_dir')))
+            self.assertEqual(mock_logger.warning.call_count, warn)
+            self.assertTrue(
+                os.path.exists(os.path.join(self.sda1, 'not_a_dir')))
             if should_skip:
                 # if we were supposed to skip over the dir, we didn't process
                 # anything at all
                 self.assertTrue(os.path.exists(prefix_dir))
                 self.assertEqual(set(), seen)
             else:
-                self.assert_(not os.path.exists(prefix_dir))
+                self.assertTrue(not os.path.exists(prefix_dir))
                 self.assertEqual(expected, seen)
 
             # test cleanup: the tempdir gets cleaned up between runs, but this
@@ -182,10 +219,10 @@ class TestObjectUpdater(unittest.TestCase):
             'concurrency': '1',
             'node_timeout': '15'})
         cu.run_once()
-        async_dir = os.path.join(self.sda1, get_async_dir(0))
+        async_dir = os.path.join(self.sda1, get_async_dir(POLICIES[0]))
         os.mkdir(async_dir)
         cu.run_once()
-        self.assert_(os.path.exists(async_dir))
+        self.assertTrue(os.path.exists(async_dir))
         # mount_check == False means no call to ismount
         self.assertEqual([], mock_ismount.mock_calls)
 
@@ -200,8 +237,8 @@ class TestObjectUpdater(unittest.TestCase):
                                'to be here')
         os.mkdir(odd_dir)
         cu.run_once()
-        self.assert_(os.path.exists(async_dir))
-        self.assert_(os.path.exists(odd_dir))  # skipped - not mounted!
+        self.assertTrue(os.path.exists(async_dir))
+        self.assertTrue(os.path.exists(odd_dir))  # skipped - not mounted!
         # mount_check == True means ismount was checked
         self.assertEqual([
             mock.call(self.sda1),
@@ -219,10 +256,10 @@ class TestObjectUpdater(unittest.TestCase):
             'concurrency': '1',
             'node_timeout': '15'}, logger=self.logger)
         cu.run_once()
-        async_dir = os.path.join(self.sda1, get_async_dir(0))
+        async_dir = os.path.join(self.sda1, get_async_dir(POLICIES[0]))
         os.mkdir(async_dir)
         cu.run_once()
-        self.assert_(os.path.exists(async_dir))
+        self.assertTrue(os.path.exists(async_dir))
         # mount_check == False means no call to ismount
         self.assertEqual([], mock_ismount.mock_calls)
 
@@ -237,8 +274,8 @@ class TestObjectUpdater(unittest.TestCase):
                                'to be here')
         os.mkdir(odd_dir)
         cu.run_once()
-        self.assert_(os.path.exists(async_dir))
-        self.assert_(not os.path.exists(odd_dir))
+        self.assertTrue(os.path.exists(async_dir))
+        self.assertTrue(not os.path.exists(odd_dir))
         # mount_check == True means ismount was checked
         self.assertEqual([
             mock.call(self.sda1),
@@ -262,14 +299,13 @@ class TestObjectUpdater(unittest.TestCase):
                                  normalize_timestamp(0)}},
                             async_pending)
         cu.run_once()
-        self.assert_(not os.path.exists(older_op_path))
-        self.assert_(os.path.exists(op_path))
+        self.assertTrue(not os.path.exists(older_op_path))
+        self.assertTrue(os.path.exists(op_path))
         self.assertEqual(cu.logger.get_increment_counts(),
                          {'failures': 1, 'unlinks': 1})
-        self.assertEqual(None,
-                         pickle.load(open(op_path)).get('successes'))
+        self.assertIsNone(pickle.load(open(op_path)).get('successes'))
 
-        bindsock = listen(('127.0.0.1', 0))
+        bindsock = listen_zero()
 
         def accepter(sock, return_code):
             try:
@@ -279,9 +315,9 @@ class TestObjectUpdater(unittest.TestCase):
                     out.write('HTTP/1.1 %d OK\r\nContent-Length: 0\r\n\r\n' %
                               return_code)
                     out.flush()
-                    self.assertEquals(inc.readline(),
-                                      'PUT /sda1/0/a/c/o HTTP/1.1\r\n')
-                    headers = swob.HeaderKeyDict()
+                    self.assertEqual(inc.readline(),
+                                     'PUT /sda1/0/a/c/o HTTP/1.1\r\n')
+                    headers = HeaderKeyDict()
                     line = inc.readline()
                     while line and line != '\r\n':
                         headers[line.split(':')[0]] = \
@@ -298,11 +334,11 @@ class TestObjectUpdater(unittest.TestCase):
             codes = iter(return_codes)
             try:
                 events = []
-                for x in xrange(len(return_codes)):
+                for x in range(len(return_codes)):
                     with Timeout(3):
                         sock, addr = bindsock.accept()
                         events.append(
-                            spawn(accepter, sock, codes.next()))
+                            spawn(accepter, sock, next(codes)))
                 for event in events:
                     err = event.wait()
                     if err:
@@ -321,22 +357,22 @@ class TestObjectUpdater(unittest.TestCase):
         err = event.wait()
         if err:
             raise err
-        self.assert_(os.path.exists(op_path))
+        self.assertTrue(os.path.exists(op_path))
         self.assertEqual(cu.logger.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0],
                          pickle.load(open(op_path)).get('successes'))
 
-        event = spawn(accept, [404, 500])
+        event = spawn(accept, [404, 201])
         cu.logger._clear()
         cu.run_once()
         err = event.wait()
         if err:
             raise err
-        self.assert_(os.path.exists(op_path))
+        self.assertTrue(os.path.exists(op_path))
         self.assertEqual(cu.logger.get_increment_counts(),
                          {'failures': 1})
-        self.assertEqual([0, 1],
+        self.assertEqual([0, 2],
                          pickle.load(open(op_path)).get('successes'))
 
         event = spawn(accept, [201])
@@ -345,7 +381,7 @@ class TestObjectUpdater(unittest.TestCase):
         err = event.wait()
         if err:
             raise err
-        self.assert_(not os.path.exists(op_path))
+        self.assertTrue(not os.path.exists(op_path))
         self.assertEqual(cu.logger.get_increment_counts(),
                          {'unlinks': 1, 'successes': 1})
 
@@ -359,7 +395,7 @@ class TestObjectUpdater(unittest.TestCase):
             'mount_check': 'false',
             'swift_dir': self.testdir,
         }
-        async_dir = os.path.join(self.sda1, get_async_dir(policy.idx))
+        async_dir = os.path.join(self.sda1, get_async_dir(policy))
         os.mkdir(async_dir)
 
         account, container, obj = 'a', 'c', 'o'
@@ -369,16 +405,16 @@ class TestObjectUpdater(unittest.TestCase):
             daemon = object_updater.ObjectUpdater(conf, logger=self.logger)
             dfmanager = DiskFileManager(conf, daemon.logger)
             # don't include storage-policy-index in headers_out pickle
-            headers_out = swob.HeaderKeyDict({
+            headers_out = HeaderKeyDict({
                 'x-size': 0,
                 'x-content-type': 'text/plain',
                 'x-etag': 'd41d8cd98f00b204e9800998ecf8427e',
-                'x-timestamp': ts.next(),
+                'x-timestamp': next(ts),
             })
             data = {'op': op, 'account': account, 'container': container,
                     'obj': obj, 'headers': headers_out}
             dfmanager.pickle_async_update(self.sda1, account, container, obj,
-                                          data, ts.next(), policy.idx)
+                                          data, next(ts), policy)
 
             request_log = []
 
@@ -394,7 +430,7 @@ class TestObjectUpdater(unittest.TestCase):
                 ip, part, method, path, headers, qs, ssl = request_args
                 self.assertEqual(method, op)
                 self.assertEqual(headers['X-Backend-Storage-Policy-Index'],
-                                 str(policy.idx))
+                                 str(int(policy)))
             self.assertEqual(daemon.logger.get_increment_counts(),
                              {'successes': 1, 'unlinks': 1,
                               'async_pendings': 1})
@@ -410,24 +446,24 @@ class TestObjectUpdater(unittest.TestCase):
             'swift_dir': self.testdir,
         }
         daemon = object_updater.ObjectUpdater(conf, logger=self.logger)
-        async_dir = os.path.join(self.sda1, get_async_dir(policy.idx))
+        async_dir = os.path.join(self.sda1, get_async_dir(policy))
         os.mkdir(async_dir)
 
         # write an async
         dfmanager = DiskFileManager(conf, daemon.logger)
         account, container, obj = 'a', 'c', 'o'
         op = 'PUT'
-        headers_out = swob.HeaderKeyDict({
+        headers_out = HeaderKeyDict({
             'x-size': 0,
             'x-content-type': 'text/plain',
             'x-etag': 'd41d8cd98f00b204e9800998ecf8427e',
-            'x-timestamp': ts.next(),
-            'X-Backend-Storage-Policy-Index': policy.idx,
+            'x-timestamp': next(ts),
+            'X-Backend-Storage-Policy-Index': int(policy),
         })
         data = {'op': op, 'account': account, 'container': container,
                 'obj': obj, 'headers': headers_out}
         dfmanager.pickle_async_update(self.sda1, account, container, obj,
-                                      data, ts.next(), policy.idx)
+                                      data, next(ts), policy)
 
         request_log = []
 
@@ -447,7 +483,7 @@ class TestObjectUpdater(unittest.TestCase):
             ip, part, method, path, headers, qs, ssl = request_args
             self.assertEqual(method, 'PUT')
             self.assertEqual(headers['X-Backend-Storage-Policy-Index'],
-                             str(policy.idx))
+                             str(int(policy)))
         self.assertEqual(daemon.logger.get_increment_counts(),
                          {'successes': 1, 'unlinks': 1, 'async_pendings': 1})
 
