@@ -16,7 +16,7 @@
 import itertools
 from collections import defaultdict
 import unittest
-from mock import patch
+import mock
 from swift.proxy.controllers.base import headers_to_container_info, \
     headers_to_account_info, headers_to_object_info, get_container_info, \
     get_cache_key, get_account_info, get_info, get_object_info, \
@@ -112,6 +112,31 @@ class DynamicResponseFactory(object):
         return resp
 
 
+class ZeroCacheAccountResponse(FakeResponse):
+    base_headers = {
+        'X-Backend-Recheck-Account-Existence': '0',
+        'x-account-container-count': 333,
+        'x-account-object-count': 1000,
+        'x-account-bytes-used': 6666,
+    }
+
+
+class ZeroCacheContainerResponse(FakeResponse):
+    base_headers = {
+        'X-Backend-Recheck-Container-Existence': '0',
+        'x-container-object-count': 1000,
+        'x-container-bytes-used': 6666,
+    }
+
+
+class ZeroCacheDynamicResponseFactory(DynamicResponseFactory):
+    response_type = {
+        'obj': ObjectResponse,
+        'container': ZeroCacheContainerResponse,
+        'account': ZeroCacheAccountResponse,
+    }
+
+
 class FakeApp(object):
 
     recheck_container_existence = 30
@@ -149,6 +174,81 @@ class TestFuncs(unittest.TestCase):
                                             account_ring=FakeRing(),
                                             container_ring=FakeRing())
 
+    def test_get_info_zero_recheck(self):
+        mock_cache = mock.Mock()
+        mock_cache.get.return_value = None
+        app = FakeApp(ZeroCacheDynamicResponseFactory())
+        env = {'swift.cache': mock_cache}
+        info_a = get_info(app, env, 'a')
+        # Check that you got proper info
+        self.assertEqual(info_a['status'], 200)
+        self.assertEqual(info_a['bytes'], 6666)
+        self.assertEqual(info_a['total_object_count'], 1000)
+        self.assertEqual(info_a['container_count'], 333)
+        # Make sure the env cache is set
+        exp_cached_info_a = {
+            k: str(v) if k in (
+                'bytes', 'container_count', 'total_object_count') else v
+            for k, v in info_a.items()}
+        self.assertEqual(env['swift.infocache'].get('account/a'),
+                         exp_cached_info_a)
+        # Make sure the app was called
+        self.assertEqual(app.responses.stats['account'], 1)
+        self.assertEqual(app.responses.stats['container'], 0)
+        # Make sure memcache was called
+        self.assertEqual(mock_cache.mock_calls, [
+            mock.call.get('account/a'),
+            mock.call.set('account/a', exp_cached_info_a, time=0),
+        ])
+
+        mock_cache.reset_mock()
+        info_c = get_info(app, env, 'a', 'c')
+        # Check that you got proper info
+        self.assertEqual(info_c['status'], 200)
+        self.assertEqual(info_c['bytes'], 6666)
+        self.assertEqual(info_c['object_count'], 1000)
+        # Make sure the env cache is set
+        exp_cached_info_c = {
+            k: str(v) if k in ('bytes', 'object_count') else v
+            for k, v in info_c.items()}
+        self.assertEqual(env['swift.infocache'].get('account/a'),
+                         exp_cached_info_a)
+        self.assertEqual(env['swift.infocache'].get('container/a/c'),
+                         exp_cached_info_c)
+        # Check app call for container, but no new calls for account
+        self.assertEqual(app.responses.stats['account'], 1)
+        self.assertEqual(app.responses.stats['container'], 1)
+        # Make sure container info was cached
+        self.assertEqual(mock_cache.mock_calls, [
+            mock.call.get('container/a/c'),
+            mock.call.set('container/a/c', exp_cached_info_c, time=0),
+        ])
+
+        # reset call counts
+        app = FakeApp(ZeroCacheDynamicResponseFactory())
+        env = {'swift.cache': mock_cache}
+        mock_cache.reset_mock()
+        info_c = get_info(app, env, 'a', 'c')
+        # Check that you got proper info
+        self.assertEqual(info_c['status'], 200)
+        self.assertEqual(info_c['bytes'], 6666)
+        self.assertEqual(info_c['object_count'], 1000)
+        # Make sure the env cache is set
+        self.assertEqual(env['swift.infocache'].get('account/a'),
+                         exp_cached_info_a)
+        self.assertEqual(env['swift.infocache'].get('container/a/c'),
+                         exp_cached_info_c)
+        # check app calls both account and container
+        self.assertEqual(app.responses.stats['account'], 1)
+        self.assertEqual(app.responses.stats['container'], 1)
+        # Make sure account info was cached but container was not
+        self.assertEqual(mock_cache.mock_calls, [
+            mock.call.get('container/a/c'),
+            mock.call.get('account/a'),
+            mock.call.set('account/a', exp_cached_info_a, time=0),
+            mock.call.set('container/a/c', exp_cached_info_c, time=0),
+        ])
+
     def test_get_info(self):
         app = FakeApp()
         # Do a non cached call to account
@@ -245,15 +345,13 @@ class TestFuncs(unittest.TestCase):
         self.assertEqual(resp['object_count'], 1000)
 
     def test_get_container_info_no_account(self):
-        responses = DynamicResponseFactory(404, 200)
-        app = FakeApp(responses)
+        app = FakeApp(statuses=[404, 200])
         req = Request.blank("/v1/AUTH_does_not_exist/cont")
         info = get_container_info(req.environ, app)
         self.assertEqual(info['status'], 0)
 
     def test_get_container_info_no_auto_account(self):
-        responses = DynamicResponseFactory(200)
-        app = FakeApp(responses)
+        app = FakeApp(statuses=[200])
         req = Request.blank("/v1/.system_account/cont")
         info = get_container_info(req.environ, app)
         self.assertEqual(info['status'], 200)
@@ -387,8 +485,8 @@ class TestFuncs(unittest.TestCase):
                             headers={'Origin': origin,
                                      'Access-Control-Request-Method': 'GET'})
 
-        with patch('swift.proxy.controllers.base.'
-                   'http_connect', fake_http_connect(200)):
+        with mock.patch('swift.proxy.controllers.base.'
+                        'http_connect', fake_http_connect(200)):
             resp = base.OPTIONS(req)
         self.assertEqual(resp.status_int, 200)
 
@@ -409,8 +507,8 @@ class TestFuncs(unittest.TestCase):
                             headers={'Origin': '*',
                                      'Access-Control-Request-Method': 'GET'})
 
-        with patch('swift.proxy.controllers.base.'
-                   'http_connect', fake_http_connect(200)):
+        with mock.patch('swift.proxy.controllers.base.'
+                        'http_connect', fake_http_connect(200)):
             resp = base.OPTIONS(req)
         self.assertEqual(resp.status_int, 200)
 
@@ -424,16 +522,16 @@ class TestFuncs(unittest.TestCase):
                             headers={'Origin': 'http://m.com',
                                      'Access-Control-Request-Method': 'GET'})
 
-        with patch('swift.proxy.controllers.base.'
-                   'http_connect', fake_http_connect(200)):
+        with mock.patch('swift.proxy.controllers.base.'
+                        'http_connect', fake_http_connect(200)):
             resp = base.OPTIONS(req)
         self.assertEqual(resp.status_int, 401)
 
     def test_headers_to_container_info_missing(self):
         resp = headers_to_container_info({}, 404)
         self.assertEqual(resp['status'], 404)
-        self.assertEqual(resp['read_acl'], None)
-        self.assertEqual(resp['write_acl'], None)
+        self.assertIsNone(resp['read_acl'])
+        self.assertIsNone(resp['write_acl'])
 
     def test_headers_to_container_info_meta(self):
         headers = {'X-Container-Meta-Whatevs': 14,
@@ -482,8 +580,8 @@ class TestFuncs(unittest.TestCase):
     def test_headers_to_account_info_missing(self):
         resp = headers_to_account_info({}, 404)
         self.assertEqual(resp['status'], 404)
-        self.assertEqual(resp['bytes'], None)
-        self.assertEqual(resp['container_count'], None)
+        self.assertIsNone(resp['bytes'])
+        self.assertIsNone(resp['container_count'])
 
     def test_headers_to_account_info_meta(self):
         headers = {'X-Account-Meta-Whatevs': 14,
@@ -519,8 +617,8 @@ class TestFuncs(unittest.TestCase):
     def test_headers_to_object_info_missing(self):
         resp = headers_to_object_info({}, 404)
         self.assertEqual(resp['status'], 404)
-        self.assertEqual(resp['length'], None)
-        self.assertEqual(resp['etag'], None)
+        self.assertIsNone(resp['length'])
+        self.assertIsNone(resp['etag'])
 
     def test_headers_to_object_info_meta(self):
         headers = {'X-Object-Meta-Whatevs': 14,
@@ -564,18 +662,18 @@ class TestFuncs(unittest.TestCase):
     def test_base_have_quorum(self):
         base = Controller(self.app)
         # just throw a bunch of test cases at it
-        self.assertEqual(base.have_quorum([201, 404], 3), False)
-        self.assertEqual(base.have_quorum([201, 201], 4), True)
-        self.assertEqual(base.have_quorum([201], 4), False)
-        self.assertEqual(base.have_quorum([201, 201, 404, 404], 4), True)
-        self.assertEqual(base.have_quorum([201, 302, 418, 503], 4), False)
-        self.assertEqual(base.have_quorum([201, 503, 503, 201], 4), True)
-        self.assertEqual(base.have_quorum([201, 201], 3), True)
-        self.assertEqual(base.have_quorum([404, 404], 3), True)
-        self.assertEqual(base.have_quorum([201, 201], 2), True)
-        self.assertEqual(base.have_quorum([201, 404], 2), True)
-        self.assertEqual(base.have_quorum([404, 404], 2), True)
-        self.assertEqual(base.have_quorum([201, 404, 201, 201], 4), True)
+        self.assertFalse(base.have_quorum([201, 404], 3))
+        self.assertTrue(base.have_quorum([201, 201], 4))
+        self.assertFalse(base.have_quorum([201], 4))
+        self.assertTrue(base.have_quorum([201, 201, 404, 404], 4))
+        self.assertFalse(base.have_quorum([201, 302, 418, 503], 4))
+        self.assertTrue(base.have_quorum([201, 503, 503, 201], 4))
+        self.assertTrue(base.have_quorum([201, 201], 3))
+        self.assertTrue(base.have_quorum([404, 404], 3))
+        self.assertTrue(base.have_quorum([201, 201], 2))
+        self.assertTrue(base.have_quorum([201, 404], 2))
+        self.assertTrue(base.have_quorum([404, 404], 2))
+        self.assertTrue(base.have_quorum([201, 404, 201, 201], 4))
 
     def test_best_response_overrides(self):
         base = Controller(self.app)
@@ -685,9 +783,9 @@ class TestFuncs(unittest.TestCase):
                             'x-base-meta-size': '151M',
                             'connection': 'close'}
         for k, v in expected_headers.items():
-            self.assertTrue(k in dst_headers)
+            self.assertIn(k, dst_headers)
             self.assertEqual(v, dst_headers[k])
-        self.assertFalse('new-owner' in dst_headers)
+        self.assertNotIn('new-owner', dst_headers)
 
     def test_generate_request_headers_with_sysmeta(self):
         base = Controller(self.app)
@@ -699,10 +797,10 @@ class TestFuncs(unittest.TestCase):
         req = Request.blank('/v1/a/c/o', headers=hdrs)
         dst_headers = base.generate_request_headers(req, transfer=True)
         for k, v in good_hdrs.items():
-            self.assertTrue(k.lower() in dst_headers)
+            self.assertIn(k.lower(), dst_headers)
             self.assertEqual(v, dst_headers[k.lower()])
         for k, v in bad_hdrs.items():
-            self.assertFalse(k.lower() in dst_headers)
+            self.assertNotIn(k.lower(), dst_headers)
 
     def test_generate_request_headers_with_no_orig_req(self):
         base = Controller(self.app)
@@ -785,8 +883,8 @@ class TestFuncs(unittest.TestCase):
             client_chunk_size=8)
 
         app_iter = handler._make_app_iter(req, node, source1)
-        with patch.object(handler, '_get_source_and_node',
-                          lambda: (source2, node)):
+        with mock.patch.object(handler, '_get_source_and_node',
+                               lambda: (source2, node)):
             client_chunks = list(app_iter)
         self.assertEqual(client_chunks, ['abcd1234', 'efgh5678'])
 
@@ -825,11 +923,49 @@ class TestFuncs(unittest.TestCase):
             client_chunk_size=8)
 
         app_iter = handler._make_app_iter(req, node, source1)
-        with patch.object(handler, '_get_source_and_node',
-                          lambda: (source2, node)):
+        with mock.patch.object(handler, '_get_source_and_node',
+                               lambda: (source2, node)):
             client_chunks = list(app_iter)
         self.assertEqual(client_chunks, ['abcd1234', 'efgh5678'])
 
+    def test_disconnected_warning(self):
+        self.app.logger = mock.Mock()
+        req = Request.blank('/v1/a/c/o')
+
+        class TestSource(object):
+            def __init__(self):
+                self.headers = {'content-type': 'text/plain',
+                                'content-length': len(self.read(-1))}
+                self.status = 200
+
+            def read(self, _read_size):
+                return 'the cake is a lie'
+
+            def getheader(self, header):
+                return self.headers.get(header.lower())
+
+            def getheaders(self):
+                return self.headers
+
+        source = TestSource()
+
+        node = {'ip': '1.2.3.4', 'port': 6200, 'device': 'sda'}
+        handler = GetOrHeadHandler(
+            self.app, req, 'Object', None, None, None, {})
+        app_iter = handler._make_app_iter(req, node, source)
+        app_iter.close()
+        self.app.logger.warning.assert_called_once_with(
+            'Client disconnected on read')
+
+        self.app.logger = mock.Mock()
+        node = {'ip': '1.2.3.4', 'port': 6200, 'device': 'sda'}
+        handler = GetOrHeadHandler(
+            self.app, req, 'Object', None, None, None, {})
+        app_iter = handler._make_app_iter(req, node, source)
+        next(app_iter)
+        app_iter.close()
+        self.app.logger.warning.assert_not_called()
+
     def test_bytes_to_skip(self):
         # if you start at the beginning, skip nothing
         self.assertEqual(bytes_to_skip(1024, 0), 0)
